import os.path
from grok.testing import register_all_tests
test_suite = register_all_tests(
    'grok.tests.testsetup.cave',
    zcml_config = os.path.join(os.path.dirname(__file__),
                               'sample.zcml'),
    layer_name = 'CustomLayerFromPath')




"""
Let's examine Grok's REST support.

Let's create a simple application with REST support::

  >>> from grok.ftests.rest.rest import MyApp
  >>> root = getRootFolder()
  >>> root['app'] = MyApp()
  >>> root['app']['alpha'] = MyContent()

Issue a GET request::

  >>> response = http_call('GET', 'http://localhost/++rest++a/app')
  >>> print response.getBody()
  GET

Issue a POST request::

  >>> response = http_call('POST', 'http://localhost/++rest++a/app')
  >>> print response.getBody()
  POST

Issue a PUT request::

  >>> response = http_call('PUT', 'http://localhost/++rest++a/app')
  >>> print response.getBody()
  PUT

Issue a DELETE request::

  >>> response = http_call('DELETE', 'http://localhost/++rest++a/app')
  >>> print response.getBody()
  DELETE

Let's examine a rest protocol b which has no POST or DELETE request defined::

The GET request works as expected::

  >>> response = http_call('GET', 'http://localhost/++rest++b/app')
  >>> print response.getBody()
  GET

So does the PUT request::

  >>> response = http_call('PUT', 'http://localhost/++rest++b/app')
  >>> print response.getBody()
  PUT

POST is not defined, however, and we should get a 405 (Method not
allowed) error::

  >>> response = http_call('POST', 'http://localhost/++rest++b/app')
  Traceback (most recent call last):
    ...
  GrokMethodNotAllowed: <grok.ftests.rest.rest.MyApp object at ...>,
  <zope.publisher.browser.BrowserRequest instance URL=http://localhost/++rest++b/app>

DELETE is also not defined, so we also expect a 405 error::

  >>> response = http_call('DELETE', 'http://localhost/++rest++b/app')
  Traceback (most recent call last):
    ...
  GrokMethodNotAllowed: <grok.ftests.rest.rest.MyApp object at ...>,
  <zope.publisher.http.HTTPRequest instance URL=http://localhost/++rest++b/app>

Let's examine protocol c where no method is allowed::

  >>> response = http_call('GET', 'http://localhost/++rest++c/app')
  Traceback (most recent call last):
    ...
  GrokMethodNotAllowed: <grok.ftests.rest.rest.MyApp object at ...
  >>> response = http_call('POST', 'http://localhost/++rest++c/app')
  Traceback (most recent call last):
    ...
  GrokMethodNotAllowed: <grok.ftests.rest.rest.MyApp object at ...
  >>> response = http_call('PUT', 'http://localhost/++rest++c/app')
  Traceback (most recent call last):
    ...
  GrokMethodNotAllowed: <grok.ftests.rest.rest.MyApp object at ...
  >>> response = http_call('DELETE', 'http://localhost/++rest++c/app')
  Traceback (most recent call last):
    ...
  GrokMethodNotAllowed: <grok.ftests.rest.rest.MyApp object at ...

Let's examine the default protocol d, where nothing should work as well::

  >>> response = http_call('GET', 'http://localhost/++rest++d/app')
  Traceback (most recent call last):
    ...
  GrokMethodNotAllowed: <grok.ftests.rest.rest.MyApp object at ...
  >>> response = http_call('POST', 'http://localhost/++rest++d/app')
  Traceback (most recent call last):
    ...
  GrokMethodNotAllowed: <grok.ftests.rest.rest.MyApp object at ...
  >>> response = http_call('PUT', 'http://localhost/++rest++d/app')
  Traceback (most recent call last):
    ...
  GrokMethodNotAllowed: <grok.ftests.rest.rest.MyApp object at ...
  >>> response = http_call('DELETE', 'http://localhost/++rest++d/app')
  Traceback (most recent call last):
    ...
  GrokMethodNotAllowed: <grok.ftests.rest.rest.MyApp object at ...

We have added support for GET for the ``alpha`` subobject only, in
the default rest layer::

  >>> response = http_call('GET', 'http://localhost/++rest++d/app/alpha')
  >>> response.getBody()
  'GET2'

But not for POST::

  >>> response = http_call('POST', 'http://localhost/++rest++d/app/alpha')
  Traceback (most recent call last):
    ...
  GrokMethodNotAllowed: <grok.ftests.rest.rest.MyContent object at ...

According to the HTTP spec, in case of a 405 Method Not Allowed error,
the response MUST include an Allow header containing a list of valid
methods for the requested resource::

  >>> print http('POST /++rest++b/app HTTP/1.1')
  HTTP/1. 405 Method Not Allowed
  Allow: GET, PUT
  Content-Length: 18
  Content-Type: text/plain
  <BLANKLINE>
  Method Not Allowed

  >>> print http('DELETE /++rest++b/app HTTP/1.1')
  HTTP/1. 405 Method Not Allowed
  Allow: GET, PUT
  Content-Length: 18
  <BLANKLINE>
  Method Not Allowed

  >>> print http('POST /++rest++c/app HTTP/1.1')
  HTTP/1. 405 Method Not Allowed
  Allow:
  Content-Length: 18
  Content-Type: text/plain
  <BLANKLINE>
  Method Not Allowed

We can also try this with a completely made-up request method, like FROG::

  >>> print http('FROG /++rest++b/app HTTP/1.1')
  HTTP/1. 405 Method Not Allowed
  Allow: GET, PUT
  Content-Length: 18
  <BLANKLINE>
  Method Not Allowed

Let's now see whether security works properly with REST. GET should
be public::

  >>> print http('GET /++rest++e/app/alpha HTTP/1.1')
  HTTP/1. 200 Ok
  Content-Length: 4
  Content-Type: text/plain
  <BLANKLINE>
  GET3

POST, PUT and DELETE however are not public::

  >>> print http('POST /++rest++e/app/alpha HTTP/1.1')
  HTTP/1. 401 Unauthorized
  Content-Length: 0
  Content-Type: text/plain
  WWW-Authenticate: basic realm="Zope"
  <BLANKLINE>

  >>> print http('PUT /++rest++e/app/alpha HTTP/1.1')
  HTTP/1. 401 Unauthorized
  Content-Length: 0
  WWW-Authenticate: basic realm="Zope"
  <BLANKLINE>

  >>> print http('DELETE /++rest++e/app/alpha HTTP/1.1')
  HTTP/1. 401 Unauthorized
  Content-Length: 0
  WWW-Authenticate: basic realm="Zope"
  <BLANKLINE>

Normally when we POST or PUT a request, we expect some content. This
content is sent along as the request body. In the normal case for POST
we tend to retrieve this information from a web form (request.form),
but with REST often the POST body contains a description of an
entirely new resource, similar to what is contained in a PUT body. We
therefore need to have some easy way to get to this information. The 'body'
attribute on the REST view contains the uploaded data::

  >>> print http_call('POST', 'http://localhost/++rest++f/app/alpha',
  ...                 'this is the POST body')
  HTTP/1.1 200 Ok
  Content-Length: 21
  Content-Type: text/plain
  <BLANKLINE>
  this is the POST body

This works with PUT as well::

  >>> print http_call('PUT', 'http://localhost/++rest++f/app/alpha',
  ...                 'this is the PUT body')
  HTTP/1.1 200 Ok
  Content-Length: 20
  <BLANKLINE>
  this is the PUT body

Opening up the publication for REST doesn't mean we can just delete
random objects without access:

  >>> print http('DELETE /app HTTP/1.1')
  HTTP/1. 500 Internal Server Error
  Content-Length: 127
  Content-Type: text/html;charset=utf-8
  <BLANKLINE>
  <html><head><title>ForbiddenAttribute</title></head>
  <body><h2>ForbiddenAttribute</h2>
  A server error occurred.
  </body></html>
  <BLANKLINE>

  >>> print http('DELETE /app/alpha HTTP/1.1')
  HTTP/1. 500 Internal Server Error
  Content-Length: 127
  Content-Type: text/html;charset=utf-8
  <BLANKLINE>
  <html><head><title>ForbiddenAttribute</title></head>
  <body><h2>ForbiddenAttribute</h2>
  A server error occurred.
  </body></html>
  <BLANKLINE>

 We shouldn't be allowed to PUT either::

  >>> print http('PUT /app/beta HTTP/1.1')
  HTTP/1. 404 Not Found
  Content-Length: 0

XXX shouldn't this really give a FORBIDDEN response?

Let's add another two pieces of content, one for which a REST view is
declared on the IFoo interface, and another one where this is also the
case, but a more specific REST view is declared on the class itself::

  >>> root['app']['one'] = MyInterfaceContent()
  >>> root['app']['two'] = MyNoInterfaceContent()

We should get a different result for the GET request::

  >>> response = http_call('GET', 'http://localhost/++rest++g/app/one')
  >>> print response.getBody()
  GET interface registered
  >>> response = http_call('GET', 'http://localhost/++rest++g/app/two')
  >>> print response.getBody()
  GET directly registered

We should also get a different result for the PUT request::

  >>> response = http_call('PUT', 'http://localhost/++rest++g/app/one')
  >>> print response.getBody()
  PUT interface registered
  >>> response = http_call('PUT', 'http://localhost/++rest++g/app/two')
  >>> print response.getBody()
  PUT directly registered

We expect POST and DELETE to be the same on both. For the directly
registered object (two) it should fall back to the interface as there
is none more specifically declared::

  >>> response = http_call('POST', 'http://localhost/++rest++g/app/one')
  >>> print response.getBody()
  POST interface registered
  >>> response = http_call('POST', 'http://localhost/++rest++g/app/two')
  >>> print response.getBody()
  POST interface registered

  >>> response = http_call('DELETE', 'http://localhost/++rest++g/app/one')
  >>> print response.getBody()
  DELETE interface registered
  >>> response = http_call('DELETE', 'http://localhost/++rest++g/app/two')
  >>> print response.getBody()
  DELETE interface registered

Todo:

* Support for OPTIONS, HEAD, other methods?

* Content-Type header is there for GET/POST, but not for PUT/DELETE...
"""

import grok
from zope.interface import Interface

class IFoo(Interface):
    pass

class MyApp(grok.Container, grok.Application):
    pass

class MyContent(grok.Model):
    pass

class LayerA(grok.IRESTLayer):
    grok.restskin('a')

class LayerB(grok.IRESTLayer):
    grok.restskin('b')

class LayerC(grok.IRESTLayer):
    grok.restskin('c')

class LayerSecurity(grok.IRESTLayer):
    grok.restskin('e')

class LayerContent(grok.IRESTLayer):
    grok.restskin('f')

class LayerInterface(grok.IRESTLayer):
    grok.restskin('g')

class D(grok.IRESTLayer):
    grok.restskin('d')

class ARest(grok.REST):
    grok.layer(LayerA)
    grok.context(MyApp)

    def GET(self):
        return "GET"

    def POST(self):
        return "POST"

    def PUT(self):
        return "PUT"

    def DELETE(self):
        return "DELETE"

class BRest(grok.REST):
    grok.layer(LayerB)
    grok.context(MyApp)

    def GET(self):
        return "GET"

    def PUT(self):
        return "PUT"

class CRest(grok.REST):
    grok.layer(LayerC)
    grok.context(MyApp)

    def some_method_thats_not_in_HTTP(self):
        pass

class DRest(grok.REST):
    grok.context(MyContent)

    def GET(self):
        return "GET2"

class SecurityRest(grok.REST):
    grok.context(MyContent)
    grok.layer(LayerSecurity)

    @grok.require(grok.Public)
    def GET(self):
        return "GET3"

    @grok.require('zope.ManageContent')
    def POST(self):
        return "POST3"

    @grok.require('zope.ManageContent')
    def PUT(self):
        return "PUT3"

    @grok.require('zope.ManageContent')
    def DELETE(self):
        return "DELETE3"

class BodyTest(grok.REST):
    grok.context(MyContent)
    grok.layer(LayerContent)

    def POST(self):
        return self.body

    def PUT(self):
        return self.body

class MyInterfaceContent(grok.Model):
    grok.implements(IFoo)

class MyNoInterfaceContent(grok.Model):
    grok.implements(IFoo)

class InterfaceRest(grok.REST):
    grok.context(IFoo)
    grok.layer(LayerInterface)

    def GET(self):
        return "GET interface registered"

    def POST(self):
        return "POST interface registered"

    def PUT(self):
        return "PUT interface registered"

    def DELETE(self):
        return "DELETE interface registered"

class NoInterfaceRest(grok.REST):
    grok.context(MyNoInterfaceContent)
    grok.layer(LayerInterface)

    def GET(self):
        return "GET directly registered"

    def PUT(self):
        return "PUT directly registered"




import grok
  
class Sample(grok.Application, grok.Container):
    pass

class Index(grok.View):
    def render(self):
        self.response.setHeader('Content-Type',
                                'text/xml; charset=UTF-8')
        return "<doc>Some XML</doc>"



"""
Models can determine how they want to be traversed by
implementing a 'traverse' method:

  >>> getRootFolder()["herd"] = Herd('The Big Mammoth Herd')

  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.handleErrors = False
  >>> browser.open("http://localhost/herd/manfred")
  >>> print browser.contents
  <html>
  <body>
  <h1>Hello, Manfred!</h1>
  <p>Manfred is part of The Big Mammoth Herd.</p>
  </body>
  </html>

  >>> browser.open("http://localhost/herd/ellie")
  >>> print browser.contents
  <html>
  <body>
  <h1>Hello, Ellie!</h1>
  <p>Ellie is part of The Big Mammoth Herd.</p>
  </body>
  </html>

"""
import grok

class Herd(grok.Model):

    def __init__(self, name):
        self.name = name

    def getMammoth(self, name):
        return Mammoth(name)

    def traverse(self, name):
        return self.getMammoth(name)
    
class Mammoth(grok.Model):

    def __init__(self, name):
        self.name = name

grok.context(Mammoth)

class Index(grok.View):
    pass

index = grok.PageTemplate("""\
<html>
<body>
<h1>Hello, <span tal:replace="context/name/title" />!</h1>
<p><span tal:replace="context/name/title" /> is part of <span tal:replace="context/__parent__/name" />.</p>
</body>
</html>
""")



##############################################################################
#
# Copyright (c) 2007 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Grok test helpers
"""
import sys
import os.path
import z3c.testsetup
from zope.configuration.config import ConfigurationMachine
from grokcore.component import zcml
# Provide this import here for BBB reasons:
from grokcore.component.testing import grok_component

class GrokTestCollector(z3c.testsetup.TestCollector):

    def initialize(self):
        # inject the grok ftesting ZCML as fallback...
        if 'zcml_config' in self.settings.keys():
            return
        pkg_path = os.path.dirname(self.package.__file__)
        if os.path.isfile(os.path.join(pkg_path, 'ftesting.zcml')):
            return
        self.settings['zcml_config'] = os.path.join(
            os.path.dirname(__file__), 'ftesting.zcml')
        if 'layer_name' in self.settings.keys():
            return
        self.settings['layer_name'] = 'GrokFunctionalLayer'

def register_all_tests(pkg, *args, **kw):
    return GrokTestCollector(pkg, *args, **kw)

def grok(module_name):
    config = ConfigurationMachine()
    zcml.do_grok('grokcore.component.meta', config)
    zcml.do_grok('grokcore.security.meta', config)
    zcml.do_grok('grokcore.view.meta', config)
    zcml.do_grok('grokcore.view.templatereg', config)
    zcml.do_grok('grokcore.viewlet.meta', config)
    zcml.do_grok('grokcore.formlib.meta', config)
    zcml.do_grok('grok.meta', config)
    zcml.do_grok(module_name, config)
    config.execute_actions()

def warn(message, category=None, stacklevel=1):
    """Intended to replace warnings.warn in tests.

    Modified copy from zope.deprecation.tests to:

      * make the signature identical to warnings.warn
      * to check for *.pyc and *.pyo files.

    When zope.deprecation is fixed, this warn function can be removed again.
    """
    print "From grok.testing's warn():"

    frame = sys._getframe(stacklevel)
    path = frame.f_globals['__file__']
    if path.endswith('.pyc') or path.endswith('.pyo'):
        path = path[:-1]

    file = open(path)
    lineno = frame.f_lineno
    for i in range(lineno):
        line = file.readline()

    print "%s:%s: %s: %s\n  %s" % (
        path,
        frame.f_lineno,
        category.__name__,
        message,
        line.strip(),
        )



"""
  >>> getRootFolder()['cave'] = cave = Cave()

JSON views answer a special content-type::

  >>> print http('GET /cave/show HTTP/1.1')
  HTTP/1. 200 Ok
  Content-Length: 17
  Content-Type: application/json
  <BLANKLINE>
  "A Cavemans cave"

"""

import grok

class Cave(grok.Model):
    pass

class CaveJSON(grok.JSON):
    def show(self):
        return 'A Cavemans cave'


"""
  >>> grok.testing.grok(__name__)
  >>>
  >>> cave = Cave()
  >>> home = IHome(cave)
  >>> IHome.providedBy(home)
  True
  >>>
  >>> isinstance(home, Home)
  True
  >>> morehome = IMoreHome(cave)
  >>> IHome.providedBy(morehome)
  True
  >>> isinstance(morehome, Home)
  True
  >>> yetanotherhome = IYetAnotherHome(cave)
  >>> IHome.providedBy(yetanotherhome)
  True
  >>> isinstance(yetanotherhome, Home)
  True

  >>> from grok.tests.adapter import noarguments_fixture
  Traceback (most recent call last):
  ...
  GrokImportError: @grok.adapter requires at least one argument.

  >>> from grok.tests.adapter import functionasargument_fixture
  Traceback (most recent call last):
  ...
  GrokImportError: @grok.adapter requires at least one argument.

"""

import grok
from zope import interface

class IDummy(interface.Interface):
    pass

class ICave(interface.Interface):
    pass

class IHome(interface.Interface):
    pass

class IMoreHome(interface.Interface):
    pass

class IYetAnotherHome(interface.Interface):
    pass

class Cave(grok.Model):
    grok.implements(ICave)
    pass

class Home(object):
    grok.implements(IHome)

@grok.adapter(Cave)
@grok.implementer(IHome)
def home_for_cave(cave):
    return Home()

@grok.adapter(ICave)
@grok.implementer(IMoreHome)
def more_home_for_cave(cave):
    return Home()

@grok.implementer(IYetAnotherHome)
def yet_another_home_for_cave(cave):
    return Home()



"""

The grok.Container is a a model that is also a container.  It has a
dictionary API. It in fact stores its information in a BTree so
you can store a lot of items in a scalable way.

    >>> grok.testing.grok(__name__)

    >>> from zope.app.container.interfaces import IContainer
    >>> bag = BoneBag()
    >>> IContainer.providedBy(bag)
    True

    >>> from zope.app.container.btree import BTreeContainer
    >>> isinstance(bag, BTreeContainer)
    True
     
We had problems when switching to grok.Container with the __parent__ attribute
being set, we better make sure this doesn't happen again:

    >>> skull = Bone()
    >>> print skull.__parent__
    None
    >>> print skull.__name__
    None
    >>> bag['skull'] = skull
    >>> skull.__parent__
    <grok.tests.container.container.BoneBag object at 0x...>
    >>> skull.__name__
    u'skull'

"""

import grok

class BoneBag(grok.Container):
    pass
    
class Bone(grok.Model):
    pass



from setuptools import setup, find_packages

version = '0.0'

setup(name='Sample',
      version=version,
      description="",
      long_description="""\
""",
      # Get strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[],
      keywords="",
      author="",
      author_email="",
      url="",
      license="",
      package_dir={'': 'src'},
      packages=find_packages('src'),
      include_package_data=True,
      zip_safe=False,
      install_requires=['setuptools',
                        'grok',
                        'grokui.admin',
                        'z3c.testsetup',
                        'grokcore.startup',
                        # Add extra requirements here
                        ],
      entry_points = """
      [console_scripts]
      sample-debug = grokcore.startup:interactive_debug_prompt
      sample-ctl = grokcore.startup:zdaemon_controller
      [paste.app_factory]
      main = grokcore.startup:application_factory
      debug = grokcore.startup:debug_application_factory
      """,
      )



"""
Subclasses of grok.Adapter and grok.MultiAdapter must implement exactly one
interface:

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
    ...
  GrokError: <class 'grok.tests.adapter.implementsnone.Home'> must
  implement at least one interface (use grok.implements to specify).
"""
import grok

class Cave(grok.Model):
    pass

class Home(grok.Adapter):
    pass



"""
A grok.ViewletManager instance has references to the components it was
registered for::

  >>> grok.testing.grok(__name__)
  >>> from zope import component
  >>> from zope.contentprovider.interfaces import IContentProvider
  >>> from zope.publisher.browser import TestRequest
  >>> ctxt = AContext()
  >>> request = TestRequest()
  >>> view = component.getMultiAdapter((ctxt, request), name='with_items')
  >>> items_mgr = component.getMultiAdapter(
  ...     (ctxt, request, view), IContentProvider, name='view_items_manager')
  >>> items_mgr.context is ctxt
  True
  >>> items_mgr.view is view
  True
  >>> items_mgr.request is request
  True

Likewise, grok.Viewlet instances have references to the components they're
registered for::

  >>> items_mgr.update()
  >>> for viewlet in items_mgr.viewlets:
  ...     viewlet.context is ctxt
  ...     viewlet.view is view
  ...     viewlet.viewletmanager is items_mgr
  ...     viewlet.request is request
  True
  True
  True
  True
  True
  True
  True
  True
"""

import grok
from zope import interface

class AContext(grok.Model):
    pass

class ViewWithItems(grok.View):
    grok.name('with_items')

    def render(self):
        return ''

class ViewItemsManager(grok.ViewletManager):
    grok.name('view_items_manager')

class ItemOneViewlet(grok.Viewlet):
    grok.name('item_one')

    def render(self):
        return "Item one reporting, sir!"

class ItemTwoViewlet(grok.Viewlet):
    grok.name('item_two')

    def render(self):
        return "Item two reporting, sir!"



"""

Permissions already set by non-grok components are preserved by the
Grok publisher.

The `@@contents.html` view of folders is protected by
`zope.ManageContent` and should not be visible to unauthenticated
users. Instead we are asked to authenticate ourselves::

  >>> print http(r'''
  ... GET /@@contents.html HTTP/1.1
  ... ''')
  HTTP/1.1 401 Unauthorized
  ...
  WWW-Authenticate: basic realm="Zope"
  ...

This is also the case for views on the Grok application object::

  >>> grok.testing.grok(__name__)
  >>> from grok.ftests.security.preserve_permissions import App
  >>> root = getRootFolder()
  >>> root['app'] = App()
  >>> print http(r'''
  ... GET /app/++etc++site HTTP/1.1
  ... ''')
  HTTP/1.1 401 Unauthorized
  ...
  WWW-Authenticate: basic realm="Zope"
  ...

We can allow our application to be viewed by the Zope standard
``contents.html`` view for site folders. For this we make it provide
`ISiteManagementFolder`::

  >>> from zope.app.component.interfaces import ISiteManagementFolder
  >>> from zope.interface import alsoProvides
  >>> alsoProvides(root['app'], ISiteManagementFolder)

Now there is a ``contents.html`` view available for our application,
which is protected by default::

  >>> print http(r'''
  ... GET /app/@@contents.html HTTP/1.1
  ... ''')
  HTTP/1.1 401 Unauthorized
  ...
  
However, if we make a grant, e.g. on the root object, we can access
the view just fine:

  >>> from zope.securitypolicy.interfaces import IPrincipalPermissionManager
  >>> root = getRootFolder()
  >>> root_perms = IPrincipalPermissionManager(root)
  >>> root_perms.grantPermissionToPrincipal('zope.ManageContent',
  ...                                       'zope.anybody')
  >>> print http(r'''
  ... GET /@@contents.html HTTP/1.1
  ... ''')
  HTTP/1.1 200 Ok
  ...

The default view is accessible::
  
  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.open('http://localhost/app')
  >>> print browser.contents
  Moo!

While the manage view is locked::

  >>> browser.open('http://localhost/app/@@manage')
  Traceback (most recent call last):
  ...
  httperror_seek_wrapper: HTTP Error 401: Unauthorized

We have some static resources defined in a local `static` directory,
which we can access unauthenticated::

  >>> browser.open('http://localhost/@@/grok.ftests.security/textfile.txt')
  >>> print browser.contents
  Just a test.

When we authenticate, everything works fine::

  >>> browser.addHeader('Authorization', 'Basic mgr:mgrpw')
  >>> browser.open('http://localhost/app/@@manage')
  >>> print browser.contents
  Woo!
  
"""
import grok

class ManageApp(grok.Permission):
    grok.name('app.Manage')

class App(grok.Application, grok.Container):
    pass

class Index(grok.View):
    def render(self):
        return 'Moo!'

class Manage(grok.View):
    grok.require('app.Manage')
    def render(self):
        return 'Woo!'



"""
Grok views provide `IGrokSecurityView` are handled more openly by the
Grok publisher.

We create an app::

  >>> root = getRootFolder()
  >>> root['app'] = App()

Now we can look at the view::

  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.handleErrors = False
  >>> browser.open('http://localhost/app/@@index')
  >>> print browser.contents
  Hello world

"""
import grok
from zope.publisher.browser import BrowserPage

class App(grok.Application, grok.Container):
    pass

class Index(grok.View):
    def render(self):
        return "Hello world"



"""
Grok allows you to set up catalog indexes in your application with a
special indexes declaration.  We do need to specify a site (such as
the application) for the Indexes however, otherwise we get a GrokError:

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
    ...
  GrokError: No site specified for grok.Indexes subclass in module
  <module 'grok.tests.catalog.indexes_no_app' from ...>.
  Use grok.site() to specify.
  
"""
import grok
from grok import index

class Herd(grok.Container, grok.Application):
    pass

class Mammoth(grok.Model):
    pass

class MammothIndexes(grok.Indexes):
    grok.context(Mammoth)
    grok.name('foo_catalog')
    
    name = index.Field()
    age = index.Field()
    message = index.Text()



import os.path
import z3c.testsetup
import sample
from zope.app.testing.functional import ZCMLLayer


ftesting_zcml = os.path.join(
    os.path.dirname(sample.__file__), 'ftesting.zcml')
FunctionalLayer = ZCMLLayer(ftesting_zcml, __name__, 'FunctionalLayer',
                            allow_teardown=True)

test_suite = z3c.testsetup.register_all_tests('sample')



"""
You cannot use local_utility with 'public' set to True if the site class
isn't a container:

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
    ...
  GrokError: Cannot set public to True with grok.local_utility as the site
  (<class 'grok.tests.utility.publicnoncontainer.Cave'>) is not a container.

"""

import grok
from zope import interface

class IFireplace(interface.Interface):
    pass

class Fireplace(grok.LocalUtility):
    grok.implements(IFireplace)
    
class Cave(grok.Model, grok.Site):
    grok.local_utility(Fireplace, public=True, name_in_container='fireplace')



"""
You can register a named adapter by using grok.name:

  >>> grok.testing.grok(__name__)

  >>> cave = Cave()
  >>> home = IHome(cave)
  Traceback (most recent call last):
    ...
  TypeError: ('Could not adapt', <grok.tests.adapter.namedadapter.Cave object at 0x...>, <InterfaceClass grok.tests.adapter.namedadapter.IHome>)

  >>> from zope.component import getAdapter
  >>> home = getAdapter(cave, IHome, name='home')
  >>> IHome.providedBy(home)
  True
  >>> isinstance(home, Home)
  True
"""

import grok
from zope import interface

class Cave(grok.Model):
    pass

class IHome(interface.Interface):
    pass

class Home(grok.Adapter):
    grok.implements(IHome)
    grok.name('home')



"""
  >>> getRootFolder()["Manfred"] = Mammoth()

  >>> from zope.app.testing.xmlrpc import ServerProxy
  >>> server = ServerProxy("http://localhost/")

  >>> server.Manfred.stomp()
  'Manfred stomped.'
  >>> server.Manfred.dance()
  "Manfred doesn't like to dance."

Let's also check whether we can use XML-RPC with subobjects we
traverse to::

  >>> server.Manfred.baby.pounce()
  'baby pounced.'
  
"""
import grok


class Mammoth(grok.Model):
    def traverse(self, name):
        if name == 'baby':
            return MammothBaby()

class MammothBaby(grok.Model):
    pass

class MammothRPC(grok.XMLRPC):
    grok.context(Mammoth)
    
    def stomp(self):
        return '%s stomped.' % self.context.__name__

    def dance(self):
        return '%s doesn\'t like to dance.' % self.context.__name__

class BabyRPC(grok.XMLRPC):
    grok.context(MammothBaby)

    def pounce(self):
        return '%s pounced.' % self.context.__name__




import grok
from zope.publisher.interfaces.browser import IBrowserRequest, IBrowserView
from zope.contentprovider.interfaces import IContentProvider
from calc import Calculator

class SingleAdapter(grok.Adapter):
    grok.context(Calculator)
    grok.adapts(Calculator)  # generally allowed, but not in this case, because there's already grok.context
    grok.implements(ISomething)  # if this is not specified, app breaks
    grok.provides(ISomething)  # if adapter implements more than one interface
    grok.name('')  # this is actually the default

    def something(self):
        """self.context is automatically provided"""
        return self.context.foo

class CalculatorContentProvider(grok.MultiAdapter)
    grok.adapts(Calculator, IBrowserRequest, IBrowserView)
    grok.implements(IContentProvider)

    def __init__(self, context, request, view):
        self.context = context
        self.request = request
        self.view = view

    # ...



"""

The grok.OrderedContainer is a a model that is also a container. Unlike plain
grok.Containers, OrderedContainers keep the mapping keys in the order items
were added. It has a dictionary API. It in fact stores its information in a
BTree so you can store a lot of items in a scalable way.

  >>> grok.testing.grok(__name__)

  >>> from zope.app.container.interfaces import IContainer
  >>> bones = OrderedBones()
  >>> IContainer.providedBy(bones)
  True
  >>> from zope.app.container.interfaces import IOrderedContainer
  >>> IOrderedContainer.providedBy(bones)
  True
  >>> from zope.app.container.btree import BTreeContainer
  >>> isinstance(bones, BTreeContainer)
  True

Order is initially determined by the sequence in which items were added (mind
that there is a subscriber to the containermodified event in order to
illustrate an ordered container fires events just like normal containers)::

  >>> bones['thigh'] = Bone('Thigh Bone')
  >>> bones['knee'] = Bone('Knee Cap')
  >>> bones['shin'] = Bone('Shin Bone')
  >>> bones['ankle'] = Bone('Ankle Joint')
  >>> bones.keys()
  ['thigh', 'knee', 'shin', 'ankle']

Now change the order::

  >>> bones.updateOrder(order=['ankle', 'shin', 'knee', 'thigh'])
  >>> bones.keys()
  ['ankle', 'shin', 'knee', 'thigh']

  >>> list(bones.items())
  [('ankle', <grok.tests.container.orderedcontainer.Bone object at ...>),
  ('shin', <grok.tests.container.orderedcontainer.Bone object at ...>),
  ('knee', <grok.tests.container.orderedcontainer.Bone object at ...>),
  ('thigh', <grok.tests.container.orderedcontainer.Bone object at ...>)]

  >>> [bone.name for bone in bones.values()]
  ['Ankle Joint', 'Shin Bone', 'Knee Cap', 'Thigh Bone']

  >>> del bones['knee']
  >>> bones.keys()
  ['ankle', 'shin', 'thigh']

  >>> bones['toe'] = Bone('Toe')
  >>> bones.keys()
  ['ankle', 'shin', 'thigh', 'toe']

Adding a new object under an existing key, raises a DuplicationError::

  >>> bones['shin'] = Bone('Another Shin Bone')
  Traceback (most recent call last):
  ...
  DuplicationError: shin

Reordering with a wrong set of keys should fail::

  >>> bones.updateOrder(order=['ankle', 'shin', 'knee', 'thigh'])
  Traceback (most recent call last):
  ...
  ValueError: Incompatible key set.

"""

import grok

class OrderedBones(grok.OrderedContainer):
    pass

class Bone(grok.Model):
    def __init__(self, name):
        self.name = name



"""
You can only use `grok.context` with interfaces or classes and not
with anything else:

  >>> function_context()
  Traceback (most recent call last):
    ...
  GrokImportError: The 'context' directive can only be called with a class or an interface.

  >>> string_context()
  Traceback (most recent call last):
    ...
  GrokImportError: The 'context' directive can only be called with a class or an interface.

  >>> module_context()
  Traceback (most recent call last):
    ...
  GrokImportError: The 'context' directive can only be called with a class or an interface.

  >>> instance_context()
  Traceback (most recent call last):
    ...
  GrokImportError: The 'context' directive can only be called with a class or an interface.

"""
import grok

def function_context():
    def a():
        pass

    class FunctionContext(object):
        grok.context(a)

def string_context():
    class StringContext(object):
        grok.context('string')

def module_context():
    class ModuleContext(object):
        grok.context(grok)

def instance_context():
    obj = object()
    class InstanceContext(object):
        grok.context(obj)



"""
Explicit class-level context for an imported model:

  >>> grok.testing.grok(__name__)

  >>> cave = Cave()
  >>> painting = IPainting(cave)

  >>> IPainting.providedBy(painting)
  True
  >>> isinstance(painting, Painting)
  True

"""
import grok
from grok.tests.adapter.adapter import Cave
from zope import interface

class IPainting(interface.Interface):
    pass

class Painting(grok.Adapter):
    grok.implements(IPainting)
    grok.context(Cave)



"""Merge versions form different buildout files.

This is a helper script for merging the Grok versions.cfg file with
the official KGS versions file.

Download the KGS versions file, for example:
http://download.zope.org/zope3.4/versions-3.4.0c1.cfg
and call it versions.kgs.

Then run this script and it will print the merged versions, with
warnings where necessary.

It picks the most recent version from both files.  If unsure, it will
warn you, in which case you need to manually edit the result.

So something like:

wget http://download.zope.org/zope3.4/versions-3.4.0c1.cfg
mv versions-3.4.0c1.cfg versions.kgs
python utilities/merge-versions.py > candidate.cfg

"""

from pkg_resources import parse_version


def make_versions(filename):
    versions_file = open(filename)
    versions = {}
    for line in versions_file.readlines():
        line = line.strip()
        split_line = line.split(' = ')
        if len(split_line) == 2:
            package = split_line[0]
            if package == 'extends':
                continue
            original = split_line[1]
            parsed = parse_version(original)
            # Store both the original and the parsed version.
            versions[package] = dict(
                original=original, parsed=parsed)
    return versions


# Hardcoded for now.
# TODO: maybe read arguments from the command line.
grok_versions = make_versions('versions.cfg')
kgs_versions = make_versions('versions.kgs')

print "[versions]"
for package, grok_version in sorted(grok_versions.items()):
    if package not in kgs_versions:
        # Extra package needed by Grok
        print "%s = %s" % (package, grok_version['original'])
    else:
        kgs_version = kgs_versions[package]
        if grok_version['parsed'] > kgs_version['parsed']:
            print "%s = %s" % (package, grok_version['original'])
        else:
            print "%s = %s" % (package, kgs_version['original'])



##############################################################################
#
# Copyright (c) 2006 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Bootstrap a buildout-based project

Simply run this script in a directory containing a buildout.cfg.
The script accepts buildout command-line options, so you can
use the -c option to specify an alternate configuration file.

$Id: bootstrap.py 102545 2009-08-06 14:49:47Z chrisw $
"""

import os, shutil, sys, tempfile, urllib2

tmpeggs = tempfile.mkdtemp()

is_jython = sys.platform.startswith('java')

try:
    import pkg_resources
except ImportError:
    ez = {}
    exec urllib2.urlopen('http://peak.telecommunity.com/dist/ez_setup.py'
                         ).read() in ez
    ez['use_setuptools'](to_dir=tmpeggs, download_delay=0)

    import pkg_resources

if sys.platform == 'win32':
    def quote(c):
        if ' ' in c:
            return '"%s"' % c # work around spawn lamosity on windows
        else:
            return c
else:
    def quote (c):
        return c

cmd = 'from setuptools.command.easy_install import main; main()'
ws  = pkg_resources.working_set

if len(sys.argv) > 2 and sys.argv[1] == '--version':
    VERSION = '==%s' % sys.argv[2]
    args = sys.argv[3:] + ['bootstrap']
else:
    VERSION = ''
    args = sys.argv[1:] + ['bootstrap']

if is_jython:
    import subprocess

    assert subprocess.Popen([sys.executable] + ['-c', quote(cmd), '-mqNxd',
           quote(tmpeggs), 'zc.buildout' + VERSION],
           env=dict(os.environ,
               PYTHONPATH=
               ws.find(pkg_resources.Requirement.parse('setuptools')).location
               ),
           ).wait() == 0

else:
    assert os.spawnle(
        os.P_WAIT, sys.executable, quote (sys.executable),
        '-c', quote (cmd), '-mqNxd', quote (tmpeggs), 'zc.buildout' + VERSION,
        dict(os.environ,
            PYTHONPATH=
            ws.find(pkg_resources.Requirement.parse('setuptools')).location
            ),
        ) == 0

ws.add_entry(tmpeggs)
ws.require('zc.buildout' + VERSION)
import zc.buildout.buildout
zc.buildout.buildout.main(args)
shutil.rmtree(tmpeggs)



"""

=============
Test Viewlets
=============

This doctest will test the various grok viewlet registrations. Grok
viewlets offer the same flexibility as zope3, allowing you to register
viewlets for a particular view, context, layer, and permission.

Set up a content object in the application root::

  >>> root = getRootFolder()
  >>> root['wilma'] = CaveWoman()
  >>> root['fred'] = CaveMan()

Traverse to the view on the model object. We get the viewlets
registered for the default layer, with the anybody permission::

  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.handleErrors = False
  >>> browser.open("http://localhost/wilma/@@caveview")
  >>> print browser.contents
  Brack Bone
  T-Rex Bone

After assigning the ``grok.BoneOwner`` role to ``zope.anybody``, he
gains the ``bone.gold`` permission. This allows the principal to view
the ``GoldBone`` viewlet::

  >>> from zope.securitypolicy.interfaces import IPrincipalRoleManager
  >>> IPrincipalRoleManager(root).assignRoleToPrincipal(
  ...    'grok.BoneOwner', 'zope.anybody')
  >>> browser.open("http://localhost/wilma/@@caveview")
  >>> print browser.contents
  Brack Bone
  Gold Bone
  T-Rex Bone

Now we traverse to the view through a skin. We gain the viewlet
registered for a particular layer, ``LayeredBone``::

  >>> browser.open('http://localhost/++skin++boneskin/wilma/@@caveview')
  >>> print browser.contents
  Brack Bone
  Gold Bone
  Layered Bone
  T-Rex Bone

Only viewlets registered for the CaveMan model, ``ManBone``, should up
when traversing to fred::

  >>> browser.open('http://localhost/fred/@@caveview')
  >>> print browser.contents
  Brack Bone
  Gold Bone
  Man Bone
  T-Rex Bone


Viewlets registered for a particular view, like ``LadyViewlet``,
should only associate with that particular one::

  >>> browser.open('http://localhost/fred/@@fireview')
  >>> print browser.contents
  Brack Bone
  Gold Bone
  Lady Viewlet
  Man Bone
  T-Rex Bone

Viewlets and viewlet managers should have a __name__. It's used
for instance when looking up the absolute url for the viewlet
or manager.

  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()
  >>> model = CaveWoman()
  >>> view = CaveView(model, request)

Managers and viewlets should get their  name from the class name
if grok.name() is not present.

  >>> manager = Pot(model, request, view)
  >>> manager.__name__
  'pot'
  >>> viewlet = BrackerBone(model, request, view, manager)
  >>> viewlet.__name__
  'brackerbone'

If grok.name() is specified for manager and viewlet they should
get those names.

  >>> manager = NamedViewletManager(model, request, view)
  >>> manager.__name__
  'managerwithname'
  >>> viewlet = NamedViewlet(model, request, view, manager)
  >>> viewlet.__name__
  'viewletwithname'

"""


import grok
from zope.interface import Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer


class Gold(grok.Permission):
    grok.name('bone.gold')

class CaveWoman(grok.Model):
    pass

class CaveMan(grok.Model):
    pass

class CaveView(grok.View):
    grok.context(Interface)

class FireView(grok.View):
    grok.context(Interface)
    grok.template('caveview')

class Pot(grok.ViewletManager):
    grok.context(Interface)
    grok.name('pot')

class TRexBone(grok.Viewlet):
    grok.context(Interface)
    grok.viewletmanager(Pot)

    def render(self):
        return "T-Rex Bone"

class BrackerBone(grok.Viewlet):
    grok.context(Interface)
    grok.viewletmanager(Pot)

    def render(self):
        return "Brack Bone"

class BoneOwner(grok.Role):
    grok.name('grok.BoneOwner')
    grok.title('Bone Ownwer')
    grok.permissions('bone.gold')

class GoldBone(grok.Viewlet):
    grok.context(Interface)
    grok.viewletmanager(Pot)
    grok.require(Gold)

    def render(self):
        return 'Gold Bone'

class IBoneLayer(IDefaultBrowserLayer):
    grok.skin('boneskin')

class LayeredBone(grok.Viewlet):
    grok.context(Interface)
    grok.viewletmanager(Pot)
    grok.layer(IBoneLayer)

    def render(self):
        return 'Layered Bone'

class ManBone(grok.Viewlet):
    grok.viewletmanager(Pot)
    grok.context(CaveMan)

    def render(self):
        return "Man Bone"

class LadyViewlet(grok.Viewlet):
    grok.context(Interface)
    grok.viewletmanager(Pot)
    grok.view(FireView)

    def render(self):
        return 'Lady Viewlet'

class NamedViewletManager(grok.ViewletManager):
    grok.context(Interface)
    grok.name('managerwithname')

class NamedViewlet(grok.Viewlet):
    grok.context(Interface)
    grok.name('viewletwithname')
    grok.viewletmanager(NamedViewletManager)

    def render(self):
        return "NamedViewlet"




"""
Local Utilities can be registered on subclasses of grok.Site using
grok.local_utility:

  >>> cave = Cave()
  >>> getRootFolder()["cave"] = cave

  >>> from zope import component
  >>> from zope.app.component.hooks import getSite, setSite
  >>> setSite(cave)

  >>> fireplace = component.getUtility(IFireplace)
  >>> IFireplace.providedBy(fireplace)
  True
  >>> isinstance(fireplace, Fireplace)
  True

  >>> club = component.getUtility(IClub)
  >>> IClub.providedBy(club)
  True
  >>> isinstance(club, Club)
  True

  >>> spiky = component.getUtility(IClub, name='spiky')
  >>> IClub.providedBy(spiky)
  True
  >>> isinstance(spiky, SpikyClub)
  True

  >>> mammoth = component.getUtility(IMammoth)
  >>> IMammoth.providedBy(mammoth)
  True
  >>> isinstance(mammoth, Mammoth)
  True

  >>> tiger = component.getUtility(IMammoth, name='tiger')
  >>> IMammoth.providedBy(tiger)
  True
  >>> isinstance(tiger, SabretoothTiger)
  True

  >>> painting = component.getUtility(IPainting, name='blackandwhite')
  >>> IPainting.providedBy(painting)
  True
  >>> isinstance(painting, CavePainting)
  True

  >>> colored = component.getUtility(IPainting, name='color')
  >>> IPainting.providedBy(colored)
  True
  >>> isinstance(colored, ColoredCavePainting)
  True

Since it is a local utility, it is not available outside its site:

  >>> setSite(None)
  >>> component.getUtility(IFireplace)
  Traceback (most recent call last):
    ...
  ComponentLookupError: (<InterfaceClass grok.ftests.utility.local.IFireplace>, '')

  >>> component.getUtility(IClub)
  Traceback (most recent call last):
    ...
  ComponentLookupError: (<InterfaceClass grok.ftests.utility.local.IClub>, '')

  >>> component.getUtility(IClub, name='spiky')
  Traceback (most recent call last):
    ...
  ComponentLookupError: (<InterfaceClass grok.ftests.utility.local.IClub>, 'spiky')

  >>> component.getUtility(IMammoth)
  Traceback (most recent call last):
    ...
  ComponentLookupError: (<InterfaceClass grok.ftests.utility.local.IMammoth>, '')

  >>> component.getUtility(IMammoth, name='tiger')
  Traceback (most recent call last):
    ...
  ComponentLookupError: (<InterfaceClass grok.ftests.utility.local.IMammoth>, 'tiger')

  >>> component.getUtility(IPainting, name='blackandwhite')
  Traceback (most recent call last):
    ...
  ComponentLookupError: (<InterfaceClass grok.ftests.utility.local.IPainting>, 'blackandwhite')

  >>> component.getUtility(IPainting, name='color')
  Traceback (most recent call last):
    ...
  ComponentLookupError: (<InterfaceClass grok.ftests.utility.local.IPainting>, 'color')
"""
import grok
from zope import interface
import persistent

class IFireplace(interface.Interface):
    pass

class IClub(interface.Interface):
    pass

class ISpiky(interface.Interface):
    pass

class IMammoth(interface.Interface):
    pass

class Fireplace(grok.LocalUtility):
    grok.implements(IFireplace)

class Club(object):
    grok.implements(IClub)

class SpikyClub(object):
    grok.implements(IClub, ISpiky)

class Mammoth(grok.LocalUtility):
    grok.implements(IMammoth, IClub)

class SabretoothTiger(grok.LocalUtility):
    grok.implements(IMammoth, IClub)
    grok.provides(IMammoth)

class IPainting(persistent.interfaces.IPersistent):
    pass

class CavePainting(grok.LocalUtility):
    grok.implements(IPainting)

class ColoredCavePainting(grok.LocalUtility):
    grok.implements(IPainting)
    grok.provides(IPainting)

class Cave(grok.Model, grok.Site):
    grok.local_utility(Fireplace)
    grok.local_utility(Club)
    grok.local_utility(SpikyClub, provides=IClub, name='spiky')
    grok.local_utility(Mammoth, provides=IMammoth)
    grok.local_utility(SabretoothTiger, name='tiger')
    grok.local_utility(CavePainting, name='blackandwhite', provides=IPainting)
    grok.local_utility(ColoredCavePainting, name='color')



##############################################################################
#
# Copyright (c) 2006-2007 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Grok utility functions.
"""
import zope.location.location
from zope import interface
from zope.security.checker import NamesChecker, defineChecker

from grokcore.security.util import check_permission

def make_checker(factory, view_factory, permission, method_names=None):
    """Make a checker for a view_factory associated with factory.

    These could be one and the same for normal views, or different
    in case we make method-based views such as for JSON and XMLRPC.
    """
    if method_names is None:
        method_names = ['__call__']
    if permission is not None:
        check_permission(factory, permission)
    if permission is None or permission == 'zope.Public':
        checker = NamesChecker(method_names)
    else:
        checker = NamesChecker(method_names, permission)
    defineChecker(view_factory, checker)

def safely_locate_maybe(obj, parent, name):
    """Set an object's __parent__ (and __name__) if the object's
    __parent__ attribute doesn't exist yet or is None.

    If the object provides ILocation, __parent__ and __name__ will be
    set directly.  A location proxy will be returned otherwise.
    """
    if getattr(obj, '__parent__', None) is not None:
        return obj
    # This either sets __parent__ or wraps 'obj' in a LocationProxy
    return zope.location.location.located(obj, parent, name)

def applySkin(request, skin, skin_type):
    """Change the presentation skin for this request.
    """
    # Remove all existing skin declarations (commonly the default skin).
    ifaces = [iface for iface in interface.directlyProvidedBy(request)
              if not skin_type.providedBy(iface)]
    # Add the new skin.
    ifaces.append(skin)
    interface.directlyProvides(request, *ifaces)



import grok
from zope import interface


grok.definelayer('my')
grok.definelayer('admin')

grok.layer('my')

grok.defineskin('my', ['my'])         # this is the default
grok.defineskin('my')                 # does the same as the line above
grok.defineskin('admin', ['admin', 'my'])


class Painting(grok.View):
    pass


fireplace = grok.PageTemplate("""\
<html><body></body></html>
""")


class AdminPainting(grok.View):
    grok.layer('adminlayer')



"""

The JSON grokker registers a view for each method of the JSON class.
So we should be able to search for view by method name.

  >>> grok.testing.grok(__name__)
  >>> mammoth = Mammoth()
  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()
  >>> from zope.component import getMultiAdapter
  >>> view = getMultiAdapter((mammoth, request), name='run')

The 'run' method/view returns json data, but it looks just like python.

  >>> view()
  '{"me": "grok"}'

Let's try calling another method::

  >>> view = getMultiAdapter((mammoth, request), name='another')
  >>> view()
  '{"another": "grok"}'

Although principally all methods of the JSON class are registered as views,
methods with names that start with an underscore are not::

  >>> view = getMultiAdapter((mammoth, request), name='_private')
  Traceback (most recent call last):
  ...
  ComponentLookupError: ((<grok.tests.json.view_lookup.Mammoth object at ...>,
  <zope.publisher.browser.TestRequest instance URL=http://127.0.0.1>),
  <InterfaceClass zope.interface.Interface>, '_private')

Even more important, special methods like __call__ are not registered
as views either. This test is here to make sure a previous bug has
been fixed::

  >>> view = getMultiAdapter((mammoth, request), name='__call__')
  Traceback (most recent call last):
  ...
  ComponentLookupError: ((<grok.tests.json.view_lookup.Mammoth object at ...>,
  <zope.publisher.browser.TestRequest instance URL=http://127.0.0.1>),
  <InterfaceClass zope.interface.Interface>, '__call__')

For JSON views we also need to confirm some methods that are defined on the
baseclass (BrowserPage) are not registered as views::

  >>> view = getMultiAdapter((mammoth, request), name='browserDefault')
  Traceback (most recent call last):
  ...
  ComponentLookupError: ((<grok.tests.json.view_lookup.Mammoth object at ...>,
  <zope.publisher.browser.TestRequest instance URL=http://127.0.0.1>),
  <InterfaceClass zope.interface.Interface>, 'browserDefault')

  >>> view = getMultiAdapter((mammoth, request), name='publishTraverse')
  Traceback (most recent call last):
  ...
  ComponentLookupError: ((<grok.tests.json.view_lookup.Mammoth object at ...>,
  <zope.publisher.browser.TestRequest instance URL=http://127.0.0.1>),
  <InterfaceClass zope.interface.Interface>, 'publishTraverse')

"""
import grok

class Mammoth(grok.Model):
    pass

class MammothView(grok.JSON):
    grok.context(Mammoth)

    def run(self):
        return { 'me': 'grok' }

    def another(self):
        return { 'another': 'grok'}

class SecondMammothView(grok.JSON):
    grok.context(Mammoth)

    def _private(self):
        return {'should': 'not be registered'}

    def public(self):
        return {'will': 'be registered'}



#############################################################################
#
# Copyright (c) 2007-2008 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Grok index definitions
"""
import sys

from zope.interface import implements
from zope.interface.interfaces import IMethod, IInterface

from zope.app.catalog.field import FieldIndex
from zope.app.catalog.text import TextIndex
from zc.catalog.catalogindex import SetIndex

from martian.error import GrokError, GrokImportError
from martian.util import frame_is_class

from grok.interfaces import IIndexDefinition

class IndexDefinition(object):
    """The definition of a particular index in a `grok.Indexes` class.

    This base class defines the actual behavior of `grok.index.Field`
    and the other kinds of attribute index that Grok supports.  Upon our
    instantiation, we save every parameter that we were passed; later,
    if an index actually needs to be created (which is typically at the
    moment when a new `grok.Application` object is added to the Zope
    Database), then our `setup()` method gets called.

    The only parameter that is actually significant to us is `attribute`
    which (optionally) defines the attribute we should index.  All other
    parameters are simply passed along to the Zope index we create,
    which interprets them as configuration details of its own.

    Note that, since index creation (and thus a call to our `setup()`
    method) currently occurs only during the creation of a new Grok
    `Application` object in the Zope Database, the presence of this
    declaration in Grok application code is nearly always a no-op.

    """
    implements(IIndexDefinition)

    index_class = None

    def __init__(self, *args, **kw):
        frame = sys._getframe(1)
        if not frame_is_class(frame):
            raise GrokImportError(
                "%r can only be instantiated on class level." % self.__class__)
        # store any extra parameters to pass to index later
        self._args = args
        self._attribute = kw.pop('attribute', None)
        self._kw = kw

    def setup(self, catalog, name, context, module_info):
        # If the user supplied attribute= when instantiating us, we
        # allow that value to override the attribute name under which we
        # are actually stored inside of the `grok.Indexes` instance.
        if self._attribute is not None:
            field_name = self._attribute
        else:
            field_name = name

        if IInterface.providedBy(context):
            try:
                method = context[field_name]
            except KeyError:
                raise GrokError("grok.Indexes in %r refers to an attribute or "
                                "method %r on interface %r, but this does not "
                                "exist." % (module_info.getModule(),
                                            field_name, context), None)
            call = IMethod.providedBy(method)
        else:
            call = callable(getattr(context, field_name, None))
            context = None # no interface lookup
        catalog[name] = self.index_class(field_name=field_name,
                                         interface=context,
                                         field_callable=call,
                                         *self._args, **self._kw)

class Field(IndexDefinition):
    """A `grok.Indexes` index that matches against an entire field."""
    index_class = FieldIndex

class Text(IndexDefinition):
    """A `grok.Indexes` index supporting full-text searches of a field."""
    index_class = TextIndex

class Set(IndexDefinition):
    """A `grok.Indexes` index supporting keyword searches of a field."""
    index_class = SetIndex



"""

We define a grokker AlphaGrokker for a component called Alpha. We first need to
grok the module defining the grokkers, in order to get them registered.

Usually this would be triggered from a meta.zcml in a package, that would grok
the module containing the grokkers (e.g. meta.py).

We do it manually now::

  >>> import grok
  >>> grok.testing.grok('grok.tests.grokker.onlyonce_fixture._meta')

This _meta.py module then will be grokked again during 'normal' grok time. Grok
will not re-register the grokkers as this could have unwanted side-effects. It
will grok the components of course.

NOTE: the module is called _meta to make sure it is grokked (although its
grokker registration should be ignored) before the other files. The modules are
picked up in alphabetical order.

To simulate this, we grok the whole package::

  >>> grok.testing.grok('grok.tests.grokker.onlyonce_fixture')
  alpha

"""



import grok.tests.grokker.priority

class AlphaSub(grok.tests.grokker.priority.Alpha):
    pass

class BetaSub(grok.tests.grokker.priority.Beta):
    pass

class GammaSub(grok.tests.grokker.priority.Gamma):
    pass



"""
Subclasses of grok.Annotation must implement at least one additional
interface to indicate which annotation interface they provide and can
be looked up with:

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
  GrokError: <class 'grok.tests.annotation.implementsnone.Branding'>
  must implement at least one interface (use grok.implements to
  specify).

"""

import grok

class Mammoth(grok.Model):
    pass

class Branding(grok.Annotation):
    pass



"""
Multiple calls of grok.require in one class are not allowed.

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
     ...
  GrokError: grok.require was called multiple times in <class 'grok.tests.xmlrpc.multiple_require.MultipleXMLRPC'>. It may only be set once for a class.
"""
import grok
import zope.interface

class One(grok.Permission):
    grok.name('permission.1')

class Two(grok.Permission):
    grok.name('permission.2')

class MultipleXMLRPC(grok.XMLRPC):
    grok.context(zope.interface.Interface)
    grok.require(One)
    grok.require(Two)

    def render(self):
        pass



##############################################################################
#
# Copyright (c) 2008 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
Tests for testsetups in grok.testing
************************************

.. warn:: Outdated!

  While all the information herein is correct, there are more and
  other options available with current version of `z3c.testsetup`.

  See http://pypi.python.org/pypi/z3c.testsetup for up-to-date
  information.

The grok testing support provides a z3c.testsetup which should suit
most grokprojects. Main parts of this support are

- a grok.testing.register_all_tests() function that registers all
  marked tests found in a package and

- a TestGetter, that is reusable in own projects.

We will discuss both in the following sections.

grok.testing.register_all_tests(<pkg>)
======================================

A function that requires a package and delivers a callable which, if
called without arguments, returns a test suite with all tests of all
doctestfiles (functional *and* unit doctests) and Python tests found
in the given package.

With this function you can setup tests like this::

   import grok
   test_suite = grok.testing.register_all_tests('cave')

In standard grokprojects, this should find all files and modules, that
have a .txt or .rst filename extension (doctests) or are regular
Python modules (Python tests). Only those tests are found, that
provide a certain marker string (see below).

To modify the default values (described below), you can pass
(optional) keyword parameters. The complete syntax is::

   grok.testing.register_all_tests(
      <pkg_or_dotted_name>,
      filter_func = <function>,
      ffilter_func = <function>,
      ufilter_func = <function>,
      pfilter_func = <function>,
      extensions = <list_of_filename_extenstions>,
      fextensions = <list_of_filename_extenstions>,
      uextensions = <list_of_filename_extenstions>
      regexp_list = <list_of_regular_expression_terms>,
      uregexp_list = <list_of_regular_expression_terms>,
      fregexp_list = <list_of_regular_expression_terms>,
      pregexp_list = <list_of_regular_expression_terms>,
      encoding = <string_with_encoding_specifier>,
      checker = <output_checker>,
      globs = <dict_of_globals>,
      fglobs = <dict_of_globals>,
      uglobs = <dict_of_globals>,
      setup = <function>,
      fsetup = <function>,
      usetup = <function>,
      teardown = <function>,
      fteardown = <function>,
      uteardown = <function>,
      optionflags = <optionflags>,
      foptionflags = <optionflags>,
      uoptionflags = <optionflags>,
      zcml_config = <path_to_zcml_config_file>,
      layer_name = <string>,
      layer = <complete_zcml_layer>)

where all parameters (except the first one) are optional. Those
options that are preceeded by 'u', 'f', or 'p' affect only unit
doctests, functional doctests or Python tests respectively.

The work of the function is done in two steps:

1) Doctestfiles and Python modules with tests are collected and

2) the tests contained in them are registered with the given values or
   default values.

We will discuss the default values and how to modify the behaviour of
both steps.


A simple test setup with register_all_tests()
---------------------------------------------

The `register_all_tests()` function must be called with a package as
first argument. This package can be given in 'dotted name' notation::

   >>> import grok
   >>> setup = grok.testing.register_all_tests(
   ...             'grok.tests.testsetup.cave')
   >>> setup
   <grok.testing.GrokTestCollector object at 0x...>

Alternatively, we can pass the 'real' package, which requires to
import it before::

   >>> from grok.tests.testsetup import cave
   >>> setup = grok.testing.register_all_tests(cave)
   >>> setup
   <grok.testing.GrokTestCollector object at 0x...>

In both cases we get a `GrokTestCollector` which is a callable, that
returns a ``unittest.TestSuite`` upon being called::

   >>> suite = setup()
   >>> suite
   <unittest.TestSuite tests=[...]>

We import a function of the z3c.testsetup to list the set of included
testfiles/modules::

   >>> from z3c.testsetup.tests.test_testsetup import get_basenames_from_suite
   >>> get_basenames_from_suite(suite)
   ['file1.py', 'file1.rst', 'file1.txt', 'subdirfile.txt']
   
These are files/modules located in the ``cave`` package. If you look
into these files, you will see, that `file1.py` is a normal Pyhton
testmodule, `file1.rst` contains unitdoctests and both .txt-files
are functional doctest files.

There are some more files in the `cave` package, which also contain
tests. Why haven't they been found? A doctestfile or module must meet
certain requirements to be accepted.


Which files/modules are found by default?
-----------------------------------------

Basically, all files/modules are found that

1) reside inside the package passed to the function. This includes
   subdirectories/subpackages.

2) have a filename extension `.txt` or `.rst` (uppercase, lowercase
   etc. does not matter) if they should be registered as unit or
   functional doctests. Python tests are always looked up in regular
   Python modules.

3) are *not* located inside a 'hidden' directory (i.e. a directory
   starting with a dot ('.'). Also subdirectories of 'hidden'
   directories are skipped. Python tests are only found, if the
   subpackage they are in is a real subpackage (the directory itself
   and all parent directories inside the root package must have an
   `__init__.py`).

4) contain a ReStructured Text meta-marker somewhere, that defines the
   file as a functional or unit doctest or Python test explicitly.

   For functional doctests, the file must contain::
   
       :Test-Layer: functional

   For unit doctests the file must contain::

       :Test-Layer: unit

   Python test modules must provide a module docstring that contains::

       :Test-Layer: python

   This means: there *must* be a line like the above one in the
   doctest file or python modules. The terms might be preceeded or
   followed by whitspace characters (spaces, tabs).

All this is the default behaviour.


Customize the set of files/modules found
----------------------------------------

If we want to grab a different set of doctest files and/or modules, we
can use three options:

1) Pass an `extensions` parameter (doctests only):

     This by default is the list ``['.txt', '.rst']. If we want to
     search .foo files too, then we can do it like this::

        >>> setup = grok.testing.register_all_tests(
        ...     cave, extensions=['.foo'])

     Note, that the behaviour is not cummulative. We have to give the
     complete list. This will result in a different set of files
     found::

        >>> suite = setup()
        >>> get_basenames_from_suite(suite)
        ['file1.py', 'notatest1.foo', 'notatest1.foo']

     This result may look surprisingly, but is correct. The `file1.py`
     module was included, because Python tests are not affected by the
     `extensions` option and `file1.py` is a Python test. The
     `notatest1.foo` file appears twice, because it was once
     registered as a unit test and once as a functional test (yes,
     this is possible although it might not make too much sense in
     reality).

     To modify only the selection of functional doctests, you can pass
     the `fextensions` option::

        >>> setup = grok.testing.register_all_tests(
        ...     cave, fextensions=['.foo'])

     Now .txt and .rst files will be searched for unit doctests and
     .foo files for functional doctests::

        >>> suite = setup()
        >>> get_basenames_from_suite(suite)
        ['file1.py', 'file1.rst', 'notatest1.foo']

     If we provide an `<x>extensions` parameter and an `extensions`
     parameter, the more specific one will override the more general
     one. If for example we use::

        >>> setup = grok.testing.register_all_tests(
        ...     cave, extensions=['.foo'], fextensions=['.txt'])

     then all unitdoctests with filename extension .foo will be found
     and all functional doctests in .txt files::

        >>> suite=setup()
        >>> get_basenames_from_suite(suite)
        ['file1.py', 'file1.txt', 'notatest1.foo', 'subdirfile.txt']

     Python tests are not affected by the `extensions` parameter. But
     to be clear: you can search for doctests (and find them!) in
     Python files. See `Doctests in Python modules`_ below.

2) Pass an `regexp_list` parameter:

     The `regexp_list` parameter determins, which terms we want to
     search for in potential doctest files and modules. By default the
     'Test-Layer' terms mentioned above are used.

     To set a required term for one specific test type (unitdoctest,
     functional doctests or Python tests), you can use the
     `uregexp_list`, `fregexp_list` and `pregexp_list` keywords
     respectively.

     If you use the `regexp_list` parameter only, all kinds of tests
     require the same term. That is normally not what you want, but
     works::

        >>> setup = grok.testing.register_all_tests(
        ...     cave, regexp_list = [':Test-Layer:'])
        >>> suite = setup()
        >>> get_basenames_from_suite(suite)
        ['file1.py', 'file1.rst', 'file1.rst', 'file1.txt',
        'file1.txt', 'subdirfile.txt', 'subdirfile.txt']

     Here the .txt and .rst files were registered twice, once as a
     unit doctest and once as a functional doctest.

     Normally you want to set a special marker for one type of
     tests. We require the unit doctest marker for functional doctests
     now::
     
        >>> setup = grok.testing.register_all_tests(
        ...     cave, fregexp_list = [':Test-Layer: unit'])
        >>> suite = setup()
        >>> get_basenames_from_suite(suite)
        ['file1.py', 'file1.rst', 'file1.rst']

     The file `file1.rst` was now registered as a functional doctest
     as well.

     As you see, regexp_lists are lists. They match (accept) a file
     iff each of the expressions of the list could be found in a line
     of a considered file.

     For example the string ':Test-Layer:' can be found in many test
     files. But the additional string 'TestCase' appears only in the
     file1.py file in the cave package::

        >>> setup = grok.testing.register_all_tests(
        ...     cave, regexp_list = ['.*:Test-Layer:.*',
        ...                          '.*TestCase.*'])
        >>> suite = setup()
        >>> get_basenames_from_suite(suite)
        ['file1.py']

3) Pass a `filter_func` parameter:

     This keyword expects an callable, which can be called with a file
     path as argument. It should return `True` or `False`. We define
     an example function::

        >>> import os.path
        >>> def myfilter(path):
        ...     if os.path.basename(path).startswith('subdir'):
        ...         return True
        ...     return False

     This function accepts all filenames that start with 'subdir',
     which matches exactly one file in our example cave. Let's see the
     results::

        >>> setup = grok.testing.register_all_tests(
        ...     cave, filter_func = myfilter)
        >>> suite = setup()
        >>> get_basenames_from_suite(suite)
        ['file1.py', 'subdirfile.txt', 'subdirfile.txt', 'subdirfile.txt']

     Again we see the one Python module. The module search is not
     affected by `filter_func`. Use `pfilter_func` instead::

        >>> def mypfilter(module):
        ...     if not hasattr(module, '__file__'):
        ...         return False
        ...     basename = os.path.basename(module.__file__)
        ...     if basename.startswith('subdirfile'):
        ...         return True
        ...     return False

        >>> setup = grok.testing.register_all_tests(
        ...     cave, pfilter_func = mypfilter)
        >>> suite = setup()
        >>> get_basenames_from_suite(suite)
        ['file1.rst', 'file1.txt', 'subdirfile.txt']

     Unit and functional dotests can be selected specifically by using
     `ufilter_func` or `ffilter_func` respectively.

     Note, that using a custom `filter_func` will disable filename
     extension filtering. You have to implement it yourself in this
     case. Therefore also the `regexp_list` parameter will have no
     effect when you define your own filter functions.


Customizing the setup of single tests
-------------------------------------

The other keyword parameters of `register_all_tests()` influence not
the set of files handled, but the manner in which each individual test
is set up.

The descriptions here will be somewhat short, because testing of such
stuff requires a more complex test setup. But the behaviour is (with
one exception described below) like the behaviour of the original
z3c.testsetup.register_all_tests() function. See

  http://svn.zope.org/z3c.testsetup/trunk/z3c/testsetup/testrunner.txt

and

  http://svn.zope.org/z3c.testsetup/trunk/z3c/testsetup/README.txt

for some examples in action.

The difference from original behaviour is the default value of the
layer for functional doctests. We will therefore start with it.

* `zcml_config`, `layer_name` and `layer`:

    `zcml_config`: a string with a filesystem path to a ZCML layer,
    that should be used during functional doctests.

    `layer_name`: a string with an arbitrary name for the file
    identified by the `zcml_config` file. The layer name is only used,
    if a `zcml_config` is set. Default is: `FunctionalLayer`.

    `layer`: a ready to use zope.testing.functional.ZCMLLayer object.

    Functional doctests need a layer to set up some framework stuff
    like registering of principals etc. The grok.register_all_tests()
    function will lookup a few places for such a file or, if the
    `zcml_config` keyword is passed, take this. The order is like
    this:

      1) if `layer` is set, take that. This overrides any
         `zcml_config` given.

      2) if `zcml_config` is set, lookup the file, setup a layer with
         that file and take that.

      3) if a file `ftesting.zcml` exists in the root of the package
         scanned for tests, register and take that.

      4) as fallback take the `ftesting.zcml` from the `grok`
         package. This is the only difference from the original
         z3c.testsetup behaviour, which takes another file as fallback
         solution.

    We will now simulate each of this four cases. For this purpose we
    will call testrunners that collect and run tests in the `cave` and
    the `cave_to_let` package.

    In `samplesetup1.py` in the cave package is a testsetup, that
    defines an own test layer and passes it as `layer` parameter. We
    dump the file contents here::

       >>> cavepath = os.path.join(os.path.dirname(__file__), 'cave')
       >>> setupfile = os.path.join(cavepath, 'samplesetup1.py')
       >>> print open(setupfile).read()
       import os.path
       from zope.app.testing.functional import ZCMLLayer
       from grok.testing import register_all_tests
       samplelayer = ZCMLLayer(
           os.path.join(os.path.dirname(__file__), 'sample.zcml'),
           __name__, 'CustomSampleLayer')
       test_suite = register_all_tests(
           'grok.tests.testsetup.cave',
           layer = samplelayer
           )

    Note, that here a custom ZCML layer is defined, based on the file
    `sample.zcml`. Now we setup a testrunner, that will read exactly
    this file. We configure it such, that it runs only functional
    tests::

       >>> import sys
       >>> old_sysargv = sys.argv # store
       >>> defaults = [
       ...     '--path', cavepath,
       ...     '--tests-pattern', '^samplesetup1$',
       ...     ]
       >>> sys.argv = 'test -f '.split()
       >>> from zope.testing import testrunner

    The testrunner is ready. Let's start it::

       >>> testrunner.run_internal(defaults)
       Running samplesetup1.CustomSampleLayer tests:
         Set up samplesetup1.CustomSampleLayer in ... seconds.
         Ran 2 tests with 0 failures and 0 errors in ... seconds.
       Tearing down left over layers:
         Tear down samplesetup1.CustomSampleLayer ... not supported
       False

    We see, that the custom layer was used. The `False` at the end
    indicates, that the testrun was finished without any failures.

    Now let's do the same, but give a path to the ZCML file instead of
    a fully configured layer. For this we use `samplesetup2` from the
    cave package::

       >>> setupfile = os.path.join(cavepath, 'samplesetup2.py')
       >>> print open(setupfile).read()
       import os.path
       from grok.testing import register_all_tests
       test_suite = register_all_tests(
           'grok.tests.testsetup.cave',
           zcml_config = os.path.join(os.path.dirname(__file__),
                                      'sample.zcml'),
           layer_name = 'CustomLayerFromPath')

    If we feed this setup to a testrunner, we get the following::

       >>> defaults = [
       ...     '--path', cavepath,
       ...     '--tests-pattern', '^samplesetup2$',
       ...     ]
       >>> sys.argv = 'test -f '.split()
       >>> testrunner.run_internal(defaults)
       Running grok.tests.testsetup.cave.CustomLayerFromPath tests:
         Set up grok.tests.testsetup.cave.CustomLayerFromPath in ... seconds.
         Ran 2 tests with 0 failures and 0 errors in ... seconds.
       Tearing down left over layers:
         Tear down grok.tests.testsetup.cave.CustomLayerFromPath ...
       not supported
       False

    Apparently the CustomLayerFromPath was found and registered.

    Now we will use the default value. The `cave` package provides an
    (empty) `ftesting.zcml` which should be found and registered, when
    no other option was given.

    The file `samplesetup3.py` will do so::

       >>> setupfile = os.path.join(cavepath, 'samplesetup3.py')
       >>> print open(setupfile).read()
       from grok.testing import register_all_tests
       test_suite = register_all_tests('grok.tests.testsetup.cave')

    This test setup has only two lines, but is perfectly valid. Will
    it register the right thing?
    
       >>> defaults = [
       ...     '--path', cavepath,
       ...     '--tests-pattern', '^samplesetup3$',
       ...     ]
       >>> sys.argv = 'test -f '.split()
       >>> testrunner.run_internal(defaults)
       Running grok.tests.testsetup.cave.FunctionalLayer tests:
         Set up grok.tests.testsetup.cave.FunctionalLayer in ... seconds.
         Ran 2 tests with 0 failures and 0 errors in ... seconds.
       Tearing down left over layers:
         Tear down grok.tests.testsetup.cave.FunctionalLayer ...
       not supported
       False

    The ftesting.zcml layer from the cave package was used
    automatically. The name `FunctionalLayer` is the default for such
    cases. It can be overriden by passing the `layer_name` keyword.

    Now, the last layer test.

    In `samplesetup4.py` in the cave package, there is another
    testsetup, which is as short as the last one, but registers tests
    for the `cave_to_let` package, which contrary to the `cave`
    package does not provide an `ftesting.zcml`. It looks like this::

       >>> setupfile = os.path.join(cavepath, 'samplesetup4.py')
       >>> print open(setupfile).read()
       from grok.testing import register_all_tests
       test_suite = register_all_tests('grok.tests.testsetup.cave_to_let')

    Obviously, the only difference is the 'cave_to_let' package
    registered. Let's run it::

       >>> defaults = [
       ...     '--path', cavepath,
       ...     '--tests-pattern', '^samplesetup4$',
       ...     ]
       >>> sys.argv = 'test -f '.split()
       >>> testrunner.run_internal(defaults)
       Running grok.tests.testsetup.cave_to_let.GrokFunctionalLayer tests:
         Set up grok.tests.testsetup.cave_to_let.GrokFunctionalLayer in ... seconds.
         Ran 1 tests with 0 failures and 0 errors in ... seconds.
       Tearing down left over layers:
         Tear down grok.tests.testsetup.cave_to_let.GrokFunctionalLayer ... not supported
       False

    So, the `GrokFunctionalLayer` was used as fallback, because the
    `cave_to_let` package has no own ftesting.zcml. Note also, that
    here the testsetup was put into a location out of the package
    tested. A testsetup does not have to be part of the package it
    tests.

       >>> sys.argv = old_sysargv # restore old values

Doctests in Python modules
==========================

or: what the heck are those 'Python tests' after all?

That a file with tests is a Python module, does not mean
automatically, that it is a Python test in the sense of
z3c.testsetup. Instead many packages contain Python files, that
contain **doctests** and not real Python tests. The file you are
reading is an example for such a module with doctests.

A real python test modules defines tests as follows::

    '''
    Tests with real TestCase objects.

    :Test-Layer: python

    '''

    import unittest

    class TestTest(unittest.TestCase):

        def setUp(self):
            pass

        def testFoo(self):
            self.assertEqual(2, 1+1)

while a doctestfile with .py filename extension might look like this::

    '''
    Doctests in a Python module.

    :Test-Layer: functional

    This file contains a lot of examples::

       >>> 1+1
       2

    Another example::

       >>> 1+2
       3
    
    '''
    # Setup stuff...
    class MyTestClass:
        pass

You can also define doctests in functions and classes of a regular
Python module. The single tests will be collected as well and setup as
doctests, as long as .py files are accepted and the file contains the
required marker string somewhere in the file.


"""



"""
A viewlet is not allowed to define its own render method and have a template
associated with it at the same time.

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
    ...
  ConfigurationExecutionError: martian.error.GrokError: Multiple possible ways to render viewlet <class 'grok.tests.viewlet.viewlet_render_and_template.Viewlet'>. It has both a 'render' method as well as an associated template.
  in:

"""

import grok
from zope.interface import Interface

class ViewletManager(grok.ViewletManager):
    grok.name('foo')
    grok.context(Interface)
    
class Viewlet(grok.Viewlet):
    grok.viewletmanager(ViewletManager)
    grok.context(Interface)
    
    def render(self):
        return "Render method but also a template!"

    



import re
import unittest
import grok
import os.path

from pkg_resources import resource_listdir
from zope.testing import doctest, renormalizing
from zope.app.testing.functional import (HTTPCaller, getRootFolder,
                                         FunctionalTestSetup, sync, ZCMLLayer)

ftesting_zcml = os.path.join(os.path.dirname(grok.__file__), 'ftesting.zcml')
GrokFunctionalLayer = ZCMLLayer(ftesting_zcml, __name__, 'GrokFunctionalLayer',
                                allow_teardown=True)

def setUp(test):
    FunctionalTestSetup().setUp()

def tearDown(test):
    FunctionalTestSetup().tearDown()

checker = renormalizing.RENormalizing([
    # Accommodate to exception wrapping in newer versions of mechanize
    (re.compile(r'httperror_seek_wrapper:', re.M), 'HTTPError:'),
    ])

def http_call(method, path, data=None, **kw):
    """Function to help make RESTful calls.

    method - HTTP method to use
    path - testbrowser style path
    data - (body) data to submit
    kw - any request parameters
    """

    if path.startswith('http://localhost'):
        path = path[len('http://localhost'):]
    request_string = '%s %s HTTP/1.1\n' % (method, path)
    for key, value in kw.items():
        request_string += '%s: %s\n' % (key, value)
    if data is not None:
        request_string += '\r\n'
        request_string += data
    return HTTPCaller()(request_string, handle_errors=False)

def suiteFromPackage(name):
    files = resource_listdir(__name__, name)
    suite = unittest.TestSuite()
    for filename in files:
        if not filename.endswith('.py'):
            continue
        if filename == '__init__.py':
            continue

        dottedname = 'grok.ftests.%s.%s' % (name, filename[:-3])
        test = doctest.DocTestSuite(
            dottedname, setUp=setUp, tearDown=tearDown,
            checker=checker,
            extraglobs=dict(http=HTTPCaller(),
                            http_call=http_call,
                            getRootFolder=getRootFolder,
                            sync=sync),
            optionflags=(doctest.ELLIPSIS+
                         doctest.NORMALIZE_WHITESPACE+
                         doctest.REPORT_NDIFF)
            )
        test.layer = GrokFunctionalLayer

        suite.addTest(test)
    return suite

def test_suite():
    suite = unittest.TestSuite()
    for name in ['xmlrpc', 'traversal', 'form', 'url', 'security',
                 'utility', 'catalog', 'site', 'rest', 'viewlet','json']:
        suite.addTest(suiteFromPackage(name))
    return suite

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



from zope.lifecycleevent.interfaces import IObjectModifiedEvent
from calc import Calculator
import grok

@grok.subscriber(Calculator, IObjectModifiedEvent)
def calculatorChanged(calc, event):
    pass


# perhaps alias zope.event.notify to grok.notify???




import martian
from component import Alpha

class AlphaGrokker(martian.ClassGrokker):
    martian.component(Alpha)
    
    def grok(self, name, factory, module_info, **kw):
        print "alpha"
        return True



"""megrok.menu

A simple but extensible menu system for Zope 3.

What is a menu?

    For web applications there are two competing design patterns that can be
    handled under the term 'menu': classic GUI menus and web site navigation.

    The essential difference in those two is the 'status' or 'context' of the
    menu. In classic GUI applications, a menu is (more or less) stateless,
    independent of the view to the application that is currently being used.

    In classic web sites, the navigation gives a navigatable overview of the
    site's hierarchy using an 'active' flag to indicate the user's position in
    the site.

    In web applications those two concepts start to mix.

    The basic implementation of the menu system takes care of the classic
    listing of options a user has in a specific context.

    An extension mechanism is used to provide status-annotation to menu items
    to show where a user currently is.

Design goals:

    -   Simple, central, menu definition without ZCML

    -   Allow submenus.

    -   Make status determination pluggable to allow for application-specific
        complex scenarios without tainting the menu system.

Menu rendering process:

    -   create set of choices (apply filters in context, permissions, condition)

    -   apply (status) annotations

Determining whether an item is active:

    a) Model based

    - find out which object belongs to the menu item

    - Is this object the same as the context of the menu?

    b) View based

    - find out which view belongs to the menu item

    - Is it the currently published view?

    Generalisation: 

Zope/Grok stuff:

    -   Menus are utilities

Questions:

    -   How to integrate with skin layers?  

    -   Aren't menus just browser views? (context, request)

    -   Two variations: allow menu items to render their HTML, or not? 

        Allow HTML:

            - provides ability to implement various menu item types

            - codes in presentation

        Don't allow HTML:

            - more flexible to customize?

        -> use views on menu items!
"""

import grok
from megrok import menu

class AddressBook(grok.Model):
    pass

class Contact(grok.Model):
    pass

class EditContact(grok.EditForm):
    grok.context(Contact)
    grok.annotation("title", _(u"Edit XY"))


layout = grok.PageTemplate("""\
<html>
<body>

    <!-- Most simple: Render the whole menu -->
    <div tal:replace="structure menu/main/@@links"/>

    <!-- Variation 1: Render whole menu as drop down list -->
    <div tal:replace="structure menu/main/@@dropdown"/>

    <!-- Variation 2: Render menu manually, get each item as link
    <ul>
        <li tal:repeat="item menu/main">
            <a tal:replace="structure item/@@link">
        </li>
    </ul>

    <!-- Variation 3: Render menu and items manually 
    <ul tal:repeat="item menu/main"><a tal:attributes="href item/action" tal:content="item/title"></ul>
    </ul>

    <!-- Variation 4: Render option elements automatically -->
    <select>
        <option tal:repeat="item menu:main" tal:replace="item/@@option">
    </select>

    <!-- Bind the menu to a different object explicitly -->
    <div tal:replace="structure python:menu['main'].bind(object1, request)"/>
    '
    <!-- Alternative: use a grok variable in the templates: -->
    <div tal:replace="structure grok/menu/main/context:asdf/"/>



    <grok:menu 
</body>
</html>""")

class IMenuItem(Interface):

    action
    title
    submenu


main = menu.Menu(
    menu.View(EditContact),         # Link to a view. Necessary information: context, name
                                    # Link to a generic action/URL
    menu.Action(_("External action"), "string:http://${context/"),
    menu.SubMenu(
        menu.Item(


############ Implementation sketches

class ViewMenuItem(object):

    def __init__(self, view):
        # View must be a view
        

class Action(object):
    
    def __init__(self, title, target):
        self.title = title
        self.target = target    # TALES expression

    def 


class BaseMenuItem(object):

    def __init__(self, condition):
        self.condition = condition

    def checkCondition(context, request):
        # - does this item apply in the given context?
        # - does the user have the necessary permissions
        # - does the user supplied condition evalute to true? 
    

class Menu(object):

    def __init__(self, menu_items):
        # XXX assert: All items must be menu items
        self.menu_items = menu_items

    def __call__(self, context, request):
        relevant_items = []

        for item in self.menu_items:
            if not item.checkCondition(context, request):
                continue

            relevant_item.append(item)





##############################################################################
#
# Copyright (c) 2007 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Grok publication factories and classes.

These factories, and the publication classes they return, make Grok
security different from the way that security normal operates during
Zope publication.  Instead of security proxies being wrapped around
every object generated during traversal, and then wrapped around the
final object before it is viewed, only a single security check is done
when Grok is in charge: a check to see whether the view selected at the
end of the traversal process is, in fact, permitted to display the
object.

"""
from grok.rest import GrokMethodNotAllowed

from zope import component
from zope.security.proxy import removeSecurityProxy
from zope.security.checker import selectChecker
from zope.publisher.publish import mapply

from zope.publisher.interfaces.browser import IBrowserView

from zope.app.publication.http import BaseHTTPPublication, HTTPPublication
from zope.app.publication.browser import BrowserPublication
from zope.app.publication.requestpublicationfactories import \
     BrowserFactory, XMLRPCFactory, HTTPFactory
from zope.app.http.interfaces import IHTTPException

from grok.interfaces import IGrokSecurityView

class ZopePublicationSansProxy(object):
    """Grok mixin that makes a publisher remove security proxies.

    This mixin overrides three methods from the `IPublication`
    interface (defined in `zope.publisher.interfaces`) to alter their
    security behavior.  The normal Zope machinery wraps a security
    proxy around the application object returned by
    `getApplication()`, and around each of the objects returned as
    `traverseName()` is then called for each URL component.  The
    versions here strip the security proxy off instead, returning the
    bare object (unless the object is a non-Grok view, in which case
    we leave the proxy installed for important security
    reasons).  Non-Grok views however, are handled like Grok views, if
    they provide `grok.interfaces.IGrokSecurityView`.

    Finally, when `callObject()` is asked to render
    the view, we quickly re-install a security proxy on the object, make
    sure that the current user is indeed allowed to invoke `__call__()`,
    then pass the bare object to the rendering machinery.

    The result is that, in place of the elaborate series of security
    checks made during the processing of a normal Zope request, Grok
    makes only a single security check: to see if the view can be
    permissibly rendered or not.

    """
    def getApplication(self, request):
        result = super(ZopePublicationSansProxy, self).getApplication(request)
        return removeSecurityProxy(result)

    def traverseName(self, request, ob, name):
        result = super(ZopePublicationSansProxy, self).traverseName(
            request, ob, name)
        bare_result = removeSecurityProxy(result)
        if IBrowserView.providedBy(bare_result):
            if IGrokSecurityView.providedBy(bare_result):
                return bare_result
            else:
                return result
        else:
            return bare_result

    def callObject(self, request, ob):
        checker = selectChecker(ob)
        if checker is not None:
            checker.check(ob, '__call__')
        return super(ZopePublicationSansProxy, self).callObject(request, ob)


class GrokBrowserPublication(ZopePublicationSansProxy, BrowserPublication):
    """Combines `BrowserPublication` with the Grok sans-proxy mixin.

    In addition to the three methods that are overridden by the
    `ZopePublicationSansProxy`, this class overrides a fourth: the
    `getDefaultTraversal()` method, which strips the security proxy from
    the object being returned by the normal method.

    """
    def getDefaultTraversal(self, request, ob):
        obj, path = super(GrokBrowserPublication, self).getDefaultTraversal(
            request, ob)
        return removeSecurityProxy(obj), path

class GrokBrowserFactory(BrowserFactory):
    """Returns the classes Grok uses for browser requests and publication.

    When an instance of this class is called, it returns a 2-element
    tuple containing:

    - The request class that Grok uses for browser requests.
    - The publication class that Grok uses to publish to a browser.

    """
    def __call__(self):
        request, publication = super(GrokBrowserFactory, self).__call__()
        return request, GrokBrowserPublication


class GrokXMLRPCPublication(ZopePublicationSansProxy, BaseHTTPPublication):
    """Combines `BaseHTTPPublication` with the Grok sans-proxy mixin."""

class GrokXMLRPCFactory(XMLRPCFactory):
    """Returns the classes Grok uses for browser requests and publication.

    When an instance of this class is called, it returns a 2-element
    tuple containing:

    - The request class that Grok uses for XML-RPC requests.
    - The publication class that Grok uses to publish to a XML-RPC.

    """
    def __call__(self):
        request, publication = super(GrokXMLRPCFactory, self).__call__()
        return request, GrokXMLRPCPublication


class GrokHTTPPublication(ZopePublicationSansProxy, HTTPPublication):
    """Combines `HTTPPublication` with the Grok sans-proxy mixin.

    Because `HTTPPublication` provides its own, special `callObject()`
    implementation, this subclass does the same, providing what is
    basically the same call (you can verify, in fact, that most of its
    lines were copied directly from the superclass's version) but with a
    few extra lines added so that - as with the simpler `callObject()`
    method in `ZopePublicationSansProxy` - it quickly places a security
    proxy around the object, makes sure that this HTTP method is
    permitted, and finally passes the bare object to the view that will
    render it.

    """
    def callObject(self, request, ob):
        orig = ob
        if not IHTTPException.providedBy(ob):
            ob = component.queryMultiAdapter((ob, request),
                                            name=request.method)
            checker = selectChecker(ob)
            if checker is not None:
                checker.check(ob, '__call__')
            ob = getattr(ob, request.method, None)
            if ob is None:
                raise GrokMethodNotAllowed(orig, request)
        return mapply(ob, request.getPositionalArguments(), request)

class GrokHTTPFactory(HTTPFactory):
    """Returns the classes Grok uses for HTTP requests and publication.

    When an instance of this class is called, it returns a 2-element
    tuple containing:

    - The request class that Grok uses for HTTP requests.
    - The publication class that Grok uses to publish to HTTP.

    """
    def __call__(self):
        request, publication = super(GrokHTTPFactory, self).__call__()
        return request, GrokHTTPPublication



"""
Grok does not depend on the alphabetical order:

  >>> grok.testing.grok(__name__)

  >>> cave = ZCave()
  >>> home = IHome(cave)

  >>> IHome.providedBy(home)
  True
  >>> isinstance(home, Home)
  True

"""
import grok
from zope import interface

class ZCave(grok.Model):
    """we call this `ZCave` because we want to test that we do not
    depend on alphabetical order"""

class IHome(interface.Interface):
    pass

class Home(grok.Adapter):
    grok.implements(IHome)



"""

Context-determination follows the same rules as for adapters. We just check
whether it's hooked up at all:

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
    ...
  GrokError: No module-level context for
  <class 'grok.tests.xmlrpc.nocontext.HomeRPC'>, please use the
  'context' directive.

"""
import grok

class HomeRPC(grok.XMLRPC):

    def foo(self):
        pass



"""
A permission has to be defined first (using grok.Permission for example)
before it can be used in @grok.require().

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
  ConfigurationExecutionError: martian.error.GrokError: Undefined permission 'doesnt.exist' in <class 'grok.tests.json.missing_permission2.MissingPermission'>. Use grok.Permission first.
  ...

"""

import grok
import zope.interface

class MissingPermission(grok.JSON):
    grok.context(zope.interface.Interface)

    @grok.require('doesnt.exist')
    def foo(self):
        pass




import grok

class Sample(grok.Application, grok.Container):
    pass

class Entry(grok.Model):
    def __init__(self, text):
        self.text = text

class SampleIndex(grok.View):
    grok.context(Sample)
    grok.name('index')

    def update(self, name=None, text=None):
        if name is None or text is None:
            return
        self.context[name] = Entry(text)

class EntryIndex(grok.View):
    grok.context(Entry)
    grok.name('index')



import grok

class Appointment(grok.Model):
    pass

class Day(grok.Model):

    def getAppointment(self, number):
        if number in self.appointments:
            return Appointment(number)
        return None # try to look up views then

    def traverse(self, name):
        return self.getAppointment(int(number))
    
class Calendar(grok.Model):
    def getYear(self, year):
        return Year(year)

    def traverse(self, name):
        return self.getYear(int(number))

# interpretation of traverse:

# * do the traverse traversal first

# * if this raises an error, propagate exception, do not swallow it (test)

# * if this returns None, fall back on "normal" traversal for the
    object (i.e. container traversal)

"""
http://.../calendar/2006/10/13/1/
           ^^^^^^^^ ^^^^ ^^ ^^ ^
            Cal.    Year  ...  A.
"""

#XXX routes (http://routes.groovie.org/) for advanced cases


# instead of traverser on the model, you can also write a separate
# traverser component:

class CalendarTraverser(grok.Traverser):
    grok.context(Calendar)  # this is actually the default
    grok.register(site=CalendarApp)  #...

    def traverse(self, name):
        now look up stuff on self.context with 'name'...
        return that



"""
An undefined permission that's never used (because it's being shadowed
by a method-level directive) doesn't raise an error:

  >>> grok.testing.grok(__name__)

"""

import grok
import zope.interface

class Permission(grok.Permission):
    grok.name('json.exists')

class MissingPermission(grok.JSON):
    grok.context(zope.interface.Interface)

    grok.require('doesnt.exist')

    @grok.require(Permission)
    def foo(self):
        pass




"""

The JSON grokker registers a view for each method of the JSON class. These
"method" views support layers and skins.

  >>> grok.testing.grok(__name__)
  >>> mammoth = Mammoth()
  >>> from zope.component import getMultiAdapter
  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()
  >>> jsonlayer_request = TestRequest(skin=IMyJSONLayer)

The 'run' view on the default skin::

  >>> view = getMultiAdapter((mammoth, request), name='run')
  >>> view()
  '{"me": "grok"}'

The 'run' on the my_json skin::

  >>> view = getMultiAdapter((mammoth, jsonlayer_request), name='run')
  >>> view()
  '{"shadows": "run on the default layer"}'

The 'another' view on the default skin, not available on the my_json::

  >>> view = getMultiAdapter((mammoth, request), name='public')
  >>> view()
  '{"public": "only availble on the default layer"}'

  >>> view = getMultiAdapter((mammoth, jsonlayer_request), name='public')
  Traceback (most recent call last):
  ...
  ComponentLookupError: ((<grok.tests.json.json_layers.Mammoth object at ...>,
  <zope.publisher.browser.TestRequest instance URL=http://127.0.0.1>),
  <InterfaceClass zope.interface.Interface>, 'public')

The 'public' view not available on the default skin, available on
the my_json skin::

  >>> view = getMultiAdapter((mammoth, request), name='another')
  Traceback (most recent call last):
  ...
  ComponentLookupError: ((<grok.tests.json.json_layers.Mammoth object at ...>,
  <zope.publisher.browser.TestRequest instance URL=http://127.0.0.1>),
  <InterfaceClass zope.interface.Interface>, 'another')

  >>> view = getMultiAdapter((mammoth, jsonlayer_request), name='another')
  >>> view()
  '{"another": "only available on my json layer"}'

Like with "normal" view, layers can aggregate view "methods"::

  >>> second_jsonlayer_request = TestRequest(skin=IMySecondJSONLayer)
  >>> view = getMultiAdapter((mammoth, second_jsonlayer_request), name='run')
  >>> view()
  '{"shadows": "run on the IMyJSONLayer layer"}'

  >>> view = getMultiAdapter(
  ...     (mammoth, second_jsonlayer_request), name='another')
  >>> view()
  '{"another": "only available on my json layer"}'

  >>> view = getMultiAdapter(
  ...     (mammoth, second_jsonlayer_request), name='yetanother')
  >>> view()
  '{"another": "only available on my second json layer"}'

"""

import grok

class IMyJSONLayer(grok.IBrowserRequest):
    grok.skin('my_json')

class Mammoth(grok.Model):
    pass

class MammothView(grok.JSON):
    grok.context(Mammoth)

    def run(self):
        return {'me': 'grok'}

    def public(self):
        return {'public': 'only availble on the default layer'}

class AnotherMammothView(grok.JSON):
    grok.context(Mammoth)
    grok.layer(IMyJSONLayer)

    def run(self):
        return {'shadows': 'run on the default layer'}

    def another(self):
        return {'another': 'only available on my json layer'}

class IMySecondJSONLayer(IMyJSONLayer):
    # Aggregates views on the first layer and adds more.
    grok.skin('my_second_json')

class YetAnotherMammothView(grok.JSON):
    grok.context(Mammoth)
    grok.layer(IMySecondJSONLayer)

    def run(self):
        return {'shadows': 'run on the IMyJSONLayer layer'}

    def yetanother(self):
        return {'another': 'only available on my second json layer'}



import grok
from zope import interface

class IHome(interface.Interface):
    pass

class IFireplace(interface.Interface):
    pass

class Fireplace(object):
    interface.implements(IHome, IFireplace)

class Cave(grok.Model, grok.Site):
    grok.local_utility(Fireplace)



"""
When you try to register multiple classes with the same (interface, name)
combination multiple times using grok.local_utility, we expect an error:

  >>> import grok.tests.utility.multiple_class_fixture
  Traceback (most recent call last):
    ...
  GrokImportError: ("Conflicting local utility registration
  <class 'grok.tests.utility.multiple_class_fixture.Fireplace2'>.
  Local utilities are registered multiple times for interface
  <InterfaceClass grok.tests.utility.multiple_class_fixture.IFireplace>
  and name 'Foo'.",
  <class 'grok.tests.utility.multiple_class_fixture.Fireplace2'>)
"""



"""
Old-style classes are also supported:

  >>> grok.testing.grok(__name__)

  >>> cave = Cave()
  >>> home = IHome(cave)

  >>> IHome.providedBy(home)
  True
  >>> isinstance(home, Home)
  True
"""
import grok
from zope import interface

class Cave:
    pass

class IHome(interface.Interface):
    pass

class Home(grok.Adapter):
    grok.implements(IHome)
    grok.context(Cave)



"""
  >>> grok.testing.grok(__name__)
  >>> from zope import component
  >>> from zope.annotation.attribute import AttributeAnnotations
  >>> component.provideAdapter(AttributeAnnotations)

If an annotation class doesn't specify anything else, its dotted name
will be used as an annotation key:

  >>> manfred = Mammoth()
  >>> ann = IImplicitName(manfred)

  >>> from zope.annotation.interfaces import IAnnotations
  >>> 'grok.tests.annotation.name.ImplicitName' in IAnnotations(manfred)
  True

Of course, annotation classes can explicity specify the name of the
annotation key that they will be stored under.  That's useful if you
want a meaningful key that's accessible from other applications and if
you want to be able to move the class around during refactorings (then
the dotted name will obviously change)

  >>> ann = IExplicitName(manfred)
  >>> 'grok.tests.annotation.name.ExplicitName' in IAnnotations(manfred)
  False
  >>> 'mammoth.branding' in IAnnotations(manfred)
  True

"""

import grok
from zope import interface
from BTrees.OOBTree import OOTreeSet

class Mammoth(grok.Model):
    pass

class IExplicitName(interface.Interface):
    pass

class IImplicitName(interface.Interface):
    pass

class ExplicitName(grok.Annotation):
    grok.implements(IExplicitName)
    grok.name('mammoth.branding')

class ImplicitName(grok.Annotation):
    grok.implements(IImplicitName)



"""
If the model is defined after the adapter, it should still be grokked
properly:

  >>> grok.testing.grok(__name__)

  >>> cave = Cave()
  >>> home = IHome(cave)

  >>> IHome.providedBy(home)
  True
  >>> isinstance(home, Home)
  True

"""
import grok
from zope import interface

class IHome(interface.Interface):
    pass

class Home(grok.Adapter):
    grok.implements(IHome)

class Cave(grok.Model):
    pass



"""
Views have an application_url() method to easily retrieve the url of the
application::

  >>> getRootFolder()['cave'] = cave = Cave()
  >>> cave['caveman'] = CaveMan()

Asking for the application_url on the cave returns the URL to the cave::

  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.handleErrors = False
  >>> browser.open('http://localhost/cave')
  >>> browser.contents
  'http://localhost/cave'
  
Asking for the application_url on the caveman returns the URL to the cave as
well::

  >>> browser.open('http://localhost/cave/caveman')
  >>> browser.contents
  'http://localhost/cave'

You can pass a name to specify a particular view or sub object to add
to the URL::

  >>> browser.open('http://localhost/cave/caveman/second')
  >>> browser.contents
  'http://localhost/cave/second'

application_url also works with empty containers::

  >>> from grok.ftests.url.application import Corridors
  >>> cave['corridors'] = Corridors()
  >>> browser.open('http://localhost/cave/corridors')
  >>> browser.contents
  'http://localhost/cave'
  
"""
import zope.interface
import grok

class IMarker(zope.interface.Interface):
    pass


class Index(grok.View):
    grok.context(IMarker)

    def render(self):
        return self.application_url()

class Second(grok.View):
    grok.context(IMarker)

    def render(self):
        return self.application_url('second')
    
class Cave(grok.Application, grok.Container):
    grok.implements(IMarker)

class CaveMan(grok.Model):
    grok.implements(IMarker)

class Corridors(grok.Container):
    grok.implements(IMarker)



import grok

class Sample(grok.Application, grok.Container):
    def __init__(self):
        super(Sample, self).__init__()
        self.list = []

    def addText(self, text):
        self.list.append(text)
        self._p_changed = True
        
class Index(grok.View):
    pass

class Edit(grok.View):
    def update(self, text=None):
        if text is None:
            return
        self.context.addText(text)
        self.redirect(self.url('index'))



"""
  >>> from zope.app.testing.xmlrpc import ServerProxy
  >>> server = ServerProxy("http://localhost/")
  >>> mgr_server = ServerProxy("http://mgr:mgrpw@localhost/")

We can access a public method just fine, but a protected method will
raise Unauthorized:

  >>> print server.stomp()
  Manfred stomped.

  >>> print server.dance()
  Traceback (most recent call last):
  ProtocolError: <ProtocolError for localhost/: 401 401 Unauthorized>

With manager privileges, the protected method is accessible, however:

  >>> print mgr_server.dance()
  Manfred doesn't like to dance.

The same applies when a default permission is defined for all XML-RPC
methods in a class:

  >>> print server.hunt()
  Traceback (most recent call last):
  ProtocolError: <ProtocolError for localhost/: 401 401 Unauthorized>

  >>> print mgr_server.hunt()
  ME GROK LIKE MAMMOTH!

  >>> print server.eat()
  MMM, MANFRED TASTE GOOD!

  >>> print server.rest()
  ME GROK TIRED!
"""
import grok
import zope.interface

class MammothRPC(grok.XMLRPC):
    grok.context(zope.interface.Interface)

    def stomp(self):
        return 'Manfred stomped.'

    @grok.require('zope.ManageContent')
    def dance(self):
        return 'Manfred doesn\'t like to dance.'

class CavemanRPC(grok.XMLRPC):
    grok.context(zope.interface.Interface)
    grok.require('zope.ManageContent')

    def hunt(self):
        return 'ME GROK LIKE MAMMOTH!'

    @grok.require('zope.View')
    def eat(self):
        return 'MMM, MANFRED TASTE GOOD!'

    @grok.require(grok.Public)
    def rest(self):
        return 'ME GROK TIRED!'



"""
You can create an index on module level, but that should lead to a GrokError:

  >>> func()
  Traceback (most recent call last):
    ...
  GrokImportError: <class 'grok.index.Field'> can only be instantiated on
  class level.
"""
from grok import index

def func():
    foo = index.Field()
    



from setuptools import setup, find_packages
import os

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

long_description = (
    read('README.txt')
    + '\n' +
    read('CHANGES.txt')
    + '\n' +
    'Download\n'
    '********\n'
    )

setup(
    name='grok',
    version = '1.0',
    author='Grok Team',
    author_email='grok-dev@zope.org',
    url='http://grok.zope.org',
    download_url='http://cheeseshop.python.org/pypi/grok/',
    description='Grok: Now even cavemen can use Zope 3!',
    long_description=long_description,
    license='ZPL',
    classifiers=['Environment :: Web Environment',
                 'Intended Audience :: Developers',
                 'License :: OSI Approved :: Zope Public License',
                 'Programming Language :: Python',
                 'Framework :: Zope3',
                 ],

    packages=find_packages('src'),
    package_dir = {'': 'src'},
    include_package_data = True,
    zip_safe=False,
    install_requires=['setuptools',
                      'martian >= 0.10, < 0.12',
                      'grokcore.component >= 1.5, < 2.0',
                      'grokcore.formlib >= 1.4',
                      'grokcore.security >= 1.1',
                      'grokcore.view >= 1.12',
                      'grokcore.viewlet >= 1.3',
                      'simplejson',
                      'pytz',
                      'ZODB3',
                      'zope.annotation',
                      'zope.app.appsetup',
                      'zope.app.authentication',
                      'zope.app.catalog',
                      'zope.app.component',
                      'zope.app.container',
                      'zope.app.folder',
                      'zope.app.intid',
                      'zope.app.keyreference',
                      'zope.app.pagetemplate',
                      'zope.app.publication',
                      'zope.app.publisher',
                      'zope.app.renderer',
                      'zope.app.security',
                      'zope.app.testing',
                      'zope.app.twisted',
                      'zope.app.zcmlfiles',
                      'zope.component',
                      'zope.configuration',
                      'zope.dottedname',
                      'zope.deprecation',
                      'zope.event',
                      'zope.formlib',
                      'zope.hookable',
                      'zope.i18nmessageid',
                      'zope.interface',
                      'zope.lifecycleevent',
                      'zope.location',
                      'zope.pagetemplate',
                      'zope.proxy',
                      'zope.publisher',
                      'zope.schema',
                      'zope.security',
                      'zope.securitypolicy',
                      'zope.testing',
                      'zope.traversing',
                      'zope.testbrowser',
                      'zope.viewlet',
                      'zc.catalog',
                      'z3c.flashmessage',
                      'z3c.autoinclude',
                      'z3c.testsetup',
                      # deprecated packages, there for backwards compatibility
                      'zope.app.error',
                      'zope.app.securitypolicy',
                      'zope.app.session',
                      ],
)



"""
Local Utilities can be registered on subclasses of grok.Site using
grok.local_utility:

  >>> cave = SpikyCave()
  >>> getRootFolder()['cave'] = cave

  >>> from zope import component
  >>> from zope.app.component.hooks import getSite, setSite
  >>> setSite(cave)

  >>> club = component.getUtility(IClub)
  >>> IClub.providedBy(club)
  True
  >>> isinstance(club, SpikyClub)
  True

  >>> list(cave.getSiteManager().keys())
  [u'SpikyClub']
"""
import grok
from zope import interface
import persistent

class IClub(interface.Interface):
    pass

class Club(grok.LocalUtility):
    grok.implements(IClub)

class SpikyClub(grok.LocalUtility):
    grok.implements(IClub)

class Cave(grok.Model, grok.Site):
    grok.local_utility(Club)

class SpikyCave(Cave):
    grok.local_utility(SpikyClub)




import grok

class Sample(grok.Application, grok.Container):
    def __init__(self):
        super(Sample, self).__init__()
        self.list = []
    
class Index(grok.View):
    pass

class Edit(grok.View):
    def update(self, text=None):
        if text is None:
            return
        # this code has a BUG!
        self.context.list.append(text)
        self.redirect(self.url('index'))



"""
When you call the grok.local_utility directive multiple times specifying
the same (interface, name) combination, we expect an error:

  >>> import grok.tests.utility.multiple_directive_fixture
  Traceback (most recent call last):
    ...
  GrokImportError: ("Conflicting local utility registration
  <class 'grok.tests.utility.multiple_directive_fixture.Fireplace2'>.
  Local utilities are registered multiple times for interface
  <InterfaceClass grok.tests.utility.multiple_directive_fixture.IFireplace>
  and name u''.",
  <class 'grok.tests.utility.multiple_directive_fixture.Fireplace2'>)
"""



"""
Non-grok views, that also provide `IGrokSecurityView` are handled more
openly by the Grok publisher.

We create an app, that provides a non-Grok view::

  >>> root = getRootFolder()
  >>> root['app'] = App()

The view must be registered first. We register it for our `app` as
context::

  >>> from zope.publisher.interfaces.browser import IDefaultBrowserLayer
  >>> from grok.ftests.security.security_view import App, Index

  >>> from zope.component import provideAdapter
  >>> from zope.interface import Interface
  >>> provideAdapter(Index, (App, IDefaultBrowserLayer), Interface, 'index')

We create a permission checker for this view, which allows everybody
to use the `__call__` method::

  >>> from grok.util import make_checker
  >>> make_checker(App, Index, None)

However, when we want to watch this view, we run into trouble::

  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.handleErrors = False
  >>> browser.open('http://localhost/app/@@index')
  Traceback (most recent call last):
  ...
  ForbiddenAttribute: ('browserDefault', <...Index object at 0x...>)

This happens, because we did not set any permissions for the
`browserDefault` method, which in 'normal' Zope3 environments means,
that access to this attribute/method is forbidden for unauthenticated
users.

Grok, however, provides a different security policy, which is less
strict in checking attribute and method permissions. This open policy
is for security reasons *not* applied to non-grok views, except, if
the view provides `IGrokSecurityView` and this way tells, that it
really wants the grok security to be applied on its methods and
attributes.

We let instances of `Index` provide `IGrokSecurityView`::

  >>> from zope.interface import classImplements
  >>> classImplements(Index, grok.interfaces.IGrokSecurityView)

Now we can watch the view::

  >>> browser.open('http://localhost/app/@@index')
  >>> print browser.contents
  Hello from index

"""
import grok
from zope.publisher.browser import BrowserPage

class App(grok.Application, grok.Container):
    pass

class Index(BrowserPage):
    def __call__(self):
        return 'Hello from index'



"""
You can't call grok.context multiple times on module level:

  >>> import grok.tests.adapter.modulecontextmultiple_fixture
  Traceback (most recent call last):
    ...
  GrokImportError: The 'context' directive can only be called once per
  class or module.

"""



"""
Subclasses of grok.Adapter and grok.MultiAdapter must implement exactly one
interface:

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
    ...
  GrokError: <class 'grok.tests.adapter.implementsmany.Home'> is implementing
  more than one interface (use grok.provides to specify which one to use).
"""
import grok

from zope import interface

class Cave(grok.Model):
    pass

class IHome(interface.Interface):
    pass

class IFireplace(interface.Interface):
    pass

class Home(grok.Adapter):
    grok.implements(IHome, IFireplace)



"""
Grok allows you to set up catalog indexes in your application with a
special indexes declaration. Here we show what happens if you try
to set up an index for an attribute that does not exist on the interface.

Let's set up a site in which we manage a couple of objects::

  >>> herd = Herd()
  >>> getRootFolder()['herd'] = herd
  Traceback (most recent call last):
    ...
  GrokError: grok.Indexes in <module
  'grok.ftests.catalog.indexes_nonexistent' from ...>
  refers to an attribute or method 'foo' on interface <InterfaceClass
  grok.ftests.catalog.indexes_nonexistent.IMammoth>, but this does not
  exist.

Nuke the catalog and intids in the end, so as not to confuse
other tests::

  >>> from zope.app.component.hooks import setSite
  >>> setSite(herd)
  >>> from zope.app.catalog.interfaces import ICatalog
  >>> from zope.component import getUtility
  >>> catalog = getUtility(ICatalog)


  >>> sm = herd.getSiteManager()
  >>> from zope.app.catalog.interfaces import ICatalog
  >>> sm.unregisterUtility(catalog, provided=ICatalog)
  True
  >>> from zope.app.intid.interfaces import IIntIds
  >>> from zope import component
  >>> intids = component.getUtility(IIntIds)
  >>> sm.unregisterUtility(intids, provided=IIntIds)
  True
"""

from zope.interface import Interface
from zope import schema

import grok
from grok import index

class Herd(grok.Container, grok.Application):
    pass

class IMammoth(Interface):
    name = schema.TextLine(title=u'Name')
    age = schema.Int(title=u'Age')
    def message():
        """Message the mammoth has for the world."""

class MammothIndexes(grok.Indexes):
    grok.site(Herd)
    grok.context(IMammoth)

    foo = index.Field()


    
